#
# Gramps - a GTK+/GNOME based genealogy program
#
# Copyright (C) 2008,2011  Gary Burton
# Copyright (C) 2010       Jakim Friant
# Copyright (C) 2011       Heinz Brinker
# Copyright (C) 2013-2016  Paul Franklin
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
#

"""Place Report"""

# ------------------------------------------------------------------------
#
# python modules
#
# ------------------------------------------------------------------------

# ------------------------------------------------------------------------
#
# gramps modules
#
# ------------------------------------------------------------------------
from gramps.gen.const import GRAMPS_LOCALE as glocale

_ = glocale.translation.sgettext
from gramps.gen.plug.menu import FilterOption, PlaceListOption, EnumeratedListOption
from gramps.gen.plug.report import Report
from gramps.gen.plug.report import MenuReportOptions
from gramps.gen.plug.report import stdoptions
from gramps.gen.plug.docgen import (
    IndexMark,
    FontStyle,
    ParagraphStyle,
    TableStyle,
    TableCellStyle,
    FONT_SANS_SERIF,
    FONT_SERIF,
    INDEX_TYPE_TOC,
    PARA_ALIGN_CENTER,
)
from gramps.gen.sort import Sort
from gramps.gen.utils.location import get_location_list
from gramps.gen.display.place import displayer as _pd
from gramps.gen.errors import ReportError
from gramps.gen.proxy import LivingProxyDb, CacheProxyDb


class PlaceReport(Report):
    """
    Place Report class
    """

    def __init__(self, database, options, user):
        """
        Create the PlaceReport object produces the Place report.

        The arguments are:

        database        - the Gramps database instance
        options         - instance of the Options class for this report
        user            - instance of a gen.user.User class

        This report needs the following parameters (class variables)
        that come in the options class.

        places          - List of places to report on.
        center          - Center of report, person or event
        incl_private    - Whether to include private data
        name_format     - Preferred format to display names
        living_people - How to handle living people
        years_past_death - Consider as living this many years after death
        """

        Report.__init__(self, database, options, user)

        self._user = user
        menu = options.menu

        self.set_locale(menu.get_option_by_name("trans").get_value())

        stdoptions.run_date_format_option(self, menu)

        stdoptions.run_private_data_option(self, menu)
        living_opt = stdoptions.run_living_people_option(self, menu, self._locale)
        self.database = CacheProxyDb(self.database)
        self._db = self.database

        self._lv = menu.get_option_by_name("living_people").get_value()
        for value, description in living_opt.get_items(xml_items=True):
            if value == self._lv:
                living_desc = self._(description)
                break
        self.living_desc = self._("(Living people: %(option_name)s)") % {
            "option_name": living_desc
        }

        places = menu.get_option_by_name("places").get_value()
        self.center = menu.get_option_by_name("center").get_value()

        stdoptions.run_name_format_option(self, menu)
        self._nd = self._name_display

        self.place_format = menu.get_option_by_name("place_format").get_value()

        filter_option = menu.get_option_by_name("filter")
        self.filter = filter_option.get_filter()

        self.sort = Sort(self._db)

        self.place_handles = []
        if self.filter.get_name() != "":
            # Use the selected filter to provide a list of place handles
            plist = self._db.iter_place_handles()
            self.place_handles = self.filter.apply(self._db, plist, user=self._user)

        if places:
            # Add places selected individually
            self.place_handles += self.__get_place_handles(places)

        if not self.place_handles:
            raise ReportError(
                _("Place Report"),
                _("Please select at least one place before running this."),
            )

        self.place_handles.sort(key=self.sort.by_place_title_key)

    def write_report(self):
        """
        The routine that actually creates the report.
        At this point, the document is opened and ready for writing.
        """

        # Write the title line. Set in INDEX marker so that this section will be
        # identified as a major category if this is included in a Book report.

        title = self._("Place Report")
        mark = IndexMark(title, INDEX_TYPE_TOC, 1)
        self.doc.start_paragraph("PLC-ReportTitle")
        self.doc.write_text(title, mark)
        self.doc.end_paragraph()
        if self._lv != LivingProxyDb.MODE_INCLUDE_ALL:
            self.doc.start_paragraph("PLC-ReportSubtitle")
            self.doc.write_text(self.living_desc)
            self.doc.end_paragraph()
        self.__write_all_places()

    def __write_all_places(self):
        """
        This procedure writes out each of the selected places.
        """
        place_nbr = 1

        with self._user.progress(
            _("Place Report"), _("Generating report"), len(self.place_handles)
        ) as step:
            for handle in self.place_handles:
                self.__write_place(handle, place_nbr)
                if self.center == "Event":
                    self.__write_referenced_events(handle)
                elif self.center == "Person":
                    self.__write_referenced_persons(handle)
                else:
                    raise AttributeError("no such center: '%s'" % self.center)
                place_nbr += 1
                # increment progress bar
                step()

    def __write_place(self, handle, place_nbr):
        """
        This procedure writes out the details of a single place
        """
        place = self._db.get_place_from_handle(handle)

        place_details = [self._("Gramps ID: %s ") % place.get_gramps_id()]
        for level in get_location_list(self._db, place):
            # Translators: needed for French, ignore otherwise
            place_details.append(
                self._("%(str1)s: %(str2)s")
                % {"str1": self._(level[1].xml_str()), "str2": level[0]}
            )

        place_names = ""
        all_names = place.get_all_names()
        if len(all_names) > 1 or __debug__:
            for place_name in all_names:
                if place_names != "":
                    # Translators: needed for Arabic, ignore otherwise
                    place_names += self._(", ")
                place_names += "%s" % place_name.get_value()
                if place_name.get_language() != "" or __debug__:
                    place_names += " (%s)" % place_name.get_language()
            place_details += [
                self._("All Names: %s", "places") % place_names,
            ]
        self.doc.start_paragraph("PLC-PlaceTitle")
        place_title = _pd.display(self._db, place, None, self.place_format)
        self.doc.write_text(
            ("%(nbr)s. %(place)s") % {"nbr": place_nbr, "place": place_title}
        )
        self.doc.end_paragraph()

        for item in place_details:
            self.doc.start_paragraph("PLC-PlaceDetails")
            self.doc.write_text(item)
            self.doc.end_paragraph()

    def __write_referenced_events(self, handle):
        """
        This procedure writes out each of the events related to the place
        """
        event_handles = [
            event_handle
            for (object_type, event_handle) in self._db.find_backlink_handles(
                handle, ["Event"]
            )
        ]
        event_handles.sort(key=self.sort.by_date_key)

        if event_handles:
            self.doc.start_paragraph("PLC-Section")
            title = self._("Events that happened at this place")
            self.doc.write_text(title)
            self.doc.end_paragraph()
            self.doc.start_table("EventTable", "PLC-EventTable")
            column_titles = [
                self._("Date"),
                self._("Type of Event"),
                self._("Person"),
                self._("Description"),
            ]
            self.doc.start_row()
            for title in column_titles:
                self.doc.start_cell("PLC-TableColumn")
                self.doc.start_paragraph("PLC-ColumnTitle")
                self.doc.write_text(title)
                self.doc.end_paragraph()
                self.doc.end_cell()
            self.doc.end_row()

        for evt_handle in event_handles:
            event = self._db.get_event_from_handle(evt_handle)
            if event:  # will be None if marked private
                date = self._get_date(event.get_date_object())
                descr = event.get_description()
                event_type = self._(self._get_type(event.get_type()))

                person_list = []
                ref_handles = [x for x in self._db.find_backlink_handles(evt_handle)]
                if not ref_handles:  # since the backlink may point to private
                    continue  # data, ignore an event with no backlinks
                for ref_type, ref_handle in ref_handles:
                    if ref_type == "Person":
                        person_list.append(ref_handle)
                    else:
                        family = self._db.get_family_from_handle(ref_handle)
                        father = family.get_father_handle()
                        if father:
                            person_list.append(father)
                        mother = family.get_mother_handle()
                        if mother:
                            person_list.append(mother)

                people = ""
                person_list = list(set(person_list))
                for p_handle in person_list:
                    person = self._db.get_person_from_handle(p_handle)
                    if person:
                        person_name = self._nd.display(person)
                        if people == "":
                            people = "%(name)s (%(id)s)" % {
                                "name": person_name,
                                "id": person.get_gramps_id(),
                            }
                        else:
                            people = self._("%(persons)s and %(name)s (%(id)s)") % {
                                "persons": people,
                                "name": person_name,
                                "id": person.get_gramps_id(),
                            }

                event_details = [date, event_type, people, descr]
                self.doc.start_row()
                for detail in event_details:
                    self.doc.start_cell("PLC-Cell")
                    self.doc.start_paragraph("PLC-Details")
                    self.doc.write_text("%s " % detail)
                    self.doc.end_paragraph()
                    self.doc.end_cell()
                self.doc.end_row()

        if event_handles:
            self.doc.end_table()

    def __write_referenced_persons(self, handle):
        """
        This procedure writes out each of the people related to the place
        """
        event_handles = [
            event_handle
            for (object_type, event_handle) in self._db.find_backlink_handles(
                handle, ["Event"]
            )
        ]

        if event_handles:
            self.doc.start_paragraph("PLC-Section")
            title = self._("People associated with this place")
            self.doc.write_text(title)
            self.doc.end_paragraph()
            self.doc.start_table("EventTable", "PLC-PersonTable")
            column_titles = [
                self._("Person"),
                self._("Type of Event"),
                self._("Description"),
                self._("Date"),
            ]
            self.doc.start_row()
            for title in column_titles:
                self.doc.start_cell("PLC-TableColumn")
                self.doc.start_paragraph("PLC-ColumnTitle")
                self.doc.write_text(title)
                self.doc.end_paragraph()
                self.doc.end_cell()
            self.doc.end_row()

        person_dict = {}
        for evt_handle in event_handles:
            ref_handles = [x for x in self._db.find_backlink_handles(evt_handle)]
            for ref_type, ref_handle in ref_handles:
                if ref_type == "Person":
                    person = self._db.get_person_from_handle(ref_handle)
                    name_entry = "%s (%s)" % (
                        self._nd.display(person),
                        person.get_gramps_id(),
                    )
                else:
                    family = self._db.get_family_from_handle(ref_handle)
                    f_handle = family.get_father_handle()
                    m_handle = family.get_mother_handle()
                    if f_handle and m_handle:
                        father = self._db.get_person_from_handle(f_handle)
                        mother = self._db.get_person_from_handle(m_handle)
                        father_name = self._nd.display(father)
                        mother_name = self._nd.display(mother)
                        father_id = father.get_gramps_id()
                        mother_id = mother.get_gramps_id()
                        name_entry = self._(
                            "%(father)s (%(father_id)s) and "
                            "%(mother)s (%(mother_id)s)"
                        ) % {
                            "father": father_name,
                            "father_id": father_id,
                            "mother": mother_name,
                            "mother_id": mother_id,
                        }
                    elif f_handle or m_handle:
                        if f_handle:
                            p_handle = f_handle
                        else:
                            p_handle = m_handle
                        person = self._db.get_person_from_handle(p_handle)

                        name_entry = "%s (%s)" % (
                            self._nd.display(person),
                            person.get_gramps_id(),
                        )
                    else:
                        # No parents - bug #7299
                        continue

                if name_entry in person_dict:
                    person_dict[name_entry].append(evt_handle)
                else:
                    person_dict[name_entry] = []
                    person_dict[name_entry].append(evt_handle)

        keys = list(person_dict.keys())
        keys.sort()

        for entry in keys:
            people = entry
            person_dict[entry].sort(key=self.sort.by_date_key)
            for evt_handle in person_dict[entry]:
                event = self._db.get_event_from_handle(evt_handle)
                if event:
                    date = self._get_date(event.get_date_object())
                    descr = event.get_description()
                    event_type = self._(self._get_type(event.get_type()))
                else:
                    date = ""
                    descr = ""
                    event_type = ""
                event_details = [people, event_type, descr, date]
                self.doc.start_row()
                for detail in event_details:
                    self.doc.start_cell("PLC-Cell")
                    self.doc.start_paragraph("PLC-Details")
                    self.doc.write_text("%s " % detail)
                    self.doc.end_paragraph()
                    self.doc.end_cell()
                people = ""  # do not repeat the name on the next event
                self.doc.end_row()

        if event_handles:
            self.doc.end_table()

    def __get_place_handles(self, places):
        """
        This procedure converts a string of place GIDs to a list of handles
        """
        place_handles = []
        for place_gid in places.split():
            place = self._db.get_place_from_gramps_id(place_gid)
            if place is not None:
                # place can be None if option is gid of other fam tree
                place_handles.append(place.get_handle())

        return place_handles


# ------------------------------------------------------------------------
#
# PlaceOptions
#
# ------------------------------------------------------------------------
class PlaceOptions(MenuReportOptions):
    """
    Defines options and provides handling interface.
    """

    def __init__(self, name, dbase):
        self.__db = dbase
        self.__filter = None
        self.__places = None
        self.__pf = None
        MenuReportOptions.__init__(self, name, dbase)

    def get_subject(self):
        """Return a string that describes the subject of the report."""
        subject = ""
        if self.__filter.get_filter().get_name():
            # Use the selected filter's name, if any
            subject += self.__filter.get_filter().get_name()
        if self.__places.get_value():
            # Add places selected individually, if any
            for place_id in self.__places.get_value().split():
                if subject:
                    subject += " + "
                place = self.__db.get_place_from_gramps_id(place_id)
                subject += _pd.display(self.__db, place, None, self.__pf.get_value())
        return subject

    def add_menu_options(self, menu):
        """
        Add options to the menu for the place report.
        """
        category_name = _("Report Options")

        # Reload filters to pick any new ones
        CustomFilters = None
        from gramps.gen.filters import CustomFilters, GenericFilter

        self.__filter = FilterOption(_("Select using filter"), 0)
        self.__filter.set_help(_("Select places using a filter"))
        filter_list = []
        filter_list.append(GenericFilter())
        filter_list.extend(CustomFilters.get_filters("Place"))
        self.__filter.set_filters(filter_list)
        menu.add_option(category_name, "filter", self.__filter)

        self.__places = PlaceListOption(_("Select places individually"))
        self.__places.set_help(_("List of places to report on"))
        menu.add_option(category_name, "places", self.__places)

        center = EnumeratedListOption(_("Center on"), "Event")
        center.set_items([("Event", _("Event")), ("Person", _("Person"))])
        center.set_help(_("If report is event or person centered"))
        menu.add_option(category_name, "center", center)

        category_name = _("Report Options (2)")

        stdoptions.add_name_format_option(menu, category_name)

        self.__pf = stdoptions.add_place_format_option(menu, category_name)

        stdoptions.add_private_data_option(menu, category_name)

        stdoptions.add_living_people_option(menu, category_name)

        locale_opt = stdoptions.add_localization_option(menu, category_name)

        stdoptions.add_date_format_option(menu, category_name, locale_opt)

    def make_default_style(self, default_style):
        """
        Make the default output style for the Place report.
        """
        self.default_style = default_style
        self.__report_title_style()
        self.__report_subtitle_style()
        self.__place_title_style()
        self.__place_details_style()
        self.__column_title_style()
        self.__section_style()
        self.__event_table_style()
        self.__details_style()
        self.__cell_style()
        self.__table_column_style()

    def __report_title_style(self):
        """
        Define the style used for the report title
        """
        font = FontStyle()
        font.set(face=FONT_SANS_SERIF, size=16, bold=1)
        para = ParagraphStyle()
        para.set_font(font)
        para.set_header_level(1)
        para.set_top_margin(0.25)
        para.set_bottom_margin(0.25)
        para.set_alignment(PARA_ALIGN_CENTER)
        para.set_description(_("The style used for the title."))
        self.default_style.add_paragraph_style("PLC-ReportTitle", para)

    def __report_subtitle_style(self):
        """
        Define the style used for the report subtitle
        """
        font = FontStyle()
        font.set(face=FONT_SANS_SERIF, size=12, bold=1)
        para = ParagraphStyle()
        para.set_font(font)
        para.set_header_level(1)
        para.set_top_margin(0.25)
        para.set_bottom_margin(0.25)
        para.set_alignment(PARA_ALIGN_CENTER)
        para.set_description(_("The style used for the subtitle."))
        self.default_style.add_paragraph_style("PLC-ReportSubtitle", para)

    def __place_title_style(self):
        """
        Define the style used for the place title
        """
        font = FontStyle()
        font.set(face=FONT_SERIF, size=12, italic=0, bold=1)
        para = ParagraphStyle()
        para.set_font(font)
        para.set(first_indent=-1.5, lmargin=1.5)
        para.set_top_margin(0.75)
        para.set_bottom_margin(0.25)
        para.set_description(_("The style used for the section headers."))
        self.default_style.add_paragraph_style("PLC-PlaceTitle", para)

    def __place_details_style(self):
        """
        Define the style used for the place details
        """
        font = FontStyle()
        font.set(face=FONT_SERIF, size=10)
        para = ParagraphStyle()
        para.set_font(font)
        para.set(first_indent=0.0, lmargin=1.5)
        para.set_description(_("The style used for details."))
        self.default_style.add_paragraph_style("PLC-PlaceDetails", para)

    def __column_title_style(self):
        """
        Define the style used for the event table column title
        """
        font = FontStyle()
        font.set(face=FONT_SERIF, size=10, bold=1)
        para = ParagraphStyle()
        para.set_font(font)
        para.set(first_indent=0.0, lmargin=0.0)
        para.set_description(_("The basic style used for table headings."))
        self.default_style.add_paragraph_style("PLC-ColumnTitle", para)

    def __section_style(self):
        """
        Define the style used for each section
        """
        font = FontStyle()
        font.set(face=FONT_SERIF, size=10, italic=0, bold=0)
        para = ParagraphStyle()
        para.set_font(font)
        para.set(first_indent=-1.5, lmargin=1.5)
        para.set_top_margin(0.5)
        para.set_bottom_margin(0.25)
        para.set_description(_("The basic style used for the text display."))
        self.default_style.add_paragraph_style("PLC-Section", para)

    def __event_table_style(self):
        """
        Define the style used for event table
        """
        table = TableStyle()
        table.set_width(100)
        table.set_columns(4)
        table.set_column_width(0, 25)
        table.set_column_width(1, 15)
        table.set_column_width(2, 35)
        table.set_column_width(3, 25)
        self.default_style.add_table_style("PLC-EventTable", table)
        table.set_width(100)
        table.set_columns(4)
        table.set_column_width(0, 35)
        table.set_column_width(1, 15)
        table.set_column_width(2, 25)
        table.set_column_width(3, 25)
        self.default_style.add_table_style("PLC-PersonTable", table)

    def __details_style(self):
        """
        Define the style used for person and event details
        """
        font = FontStyle()
        font.set(face=FONT_SERIF, size=10)
        para = ParagraphStyle()
        para.set_font(font)
        para.set_description(_("The style used for the items and values."))
        self.default_style.add_paragraph_style("PLC-Details", para)

    def __cell_style(self):
        """
        Define the style used for cells in the event table
        """
        cell = TableCellStyle()
        self.default_style.add_cell_style("PLC-Cell", cell)

    def __table_column_style(self):
        """
        Define the style used for event table columns
        """
        cell = TableCellStyle()
        cell.set_bottom_border(1)
        self.default_style.add_cell_style("PLC-TableColumn", cell)



# -*- coding: utf-8 -*-
#
# Gramps - a GTK+/GNOME based genealogy program
#
# Copyright (C) 2013 Vassilii Khachaturov <vassilii@tarunz.org>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
#

"""Unittest for user.py"""

import unittest
from unittest.mock import Mock, patch
from .. import user
import sys


class TestUser:
    TITLE = "Testing prompt"
    MSG = "Choices are hard. Nevertheless, please choose!"
    ACCEPT = "To be"
    REJECT = "Not to be"


class TestUser_prompt(unittest.TestCase):
    def setUp(self):
        self.real_user = user.User()
        self.user = user.User()
        self.user._fileout = Mock(spec=sys.stderr)
        self.user._input = Mock(spec=input)

    def test_default_fileout_has_write(self):
        self.assertTrue(hasattr(self.real_user._fileout, "write"))

    def test_default_input(self):
        self.assertTrue(self.real_user._input.__name__.endswith("input"))

    def test_prompt_returns_True_if_ACCEPT_entered(self):
        self.user._input.configure_mock(return_value=TestUser.ACCEPT)
        self.assertTrue(
            self.user.prompt(
                TestUser.TITLE, TestUser.MSG, TestUser.ACCEPT, TestUser.REJECT
            ),
            "True expected!",
        )
        self.user._input.assert_called_once_with()

    def test_prompt_returns_False_if_REJECT_entered(self):
        self.user._input.configure_mock(return_value=TestUser.REJECT)
        self.assertFalse(
            self.user.prompt(
                TestUser.TITLE, TestUser.MSG, TestUser.ACCEPT, TestUser.REJECT
            ),
            "False expected!",
        )
        self.user._input.assert_called_once_with()

    def assert_prompt_contains_text(
        self,
        text,
        title=TestUser.TITLE,
        msg=TestUser.MSG,
        accept=TestUser.ACCEPT,
        reject=TestUser.REJECT,
    ):
        self.user._input.configure_mock(return_value=TestUser.REJECT)
        self.user.prompt(title, msg, accept, reject)
        for call in self.user._fileout.method_calls:
            name, args, kwargs = call
            for a in args:
                if a.find(text) >= 0:
                    return
        self.assertTrue(
            False,
            "'{}' never printed in prompt: {}".format(
                text, self.user._fileout.method_calls
            ),
        )

    def test_prompt_contains_title_text(self):
        self.assert_prompt_contains_text(TestUser.TITLE)

    def test_prompt_contains_msg_text(self):
        self.assert_prompt_contains_text(TestUser.MSG)

    def test_prompt_contains_accept_text(self):
        self.assert_prompt_contains_text(TestUser.ACCEPT)

    def test_prompt_contains_reject_text(self):
        self.assert_prompt_contains_text(TestUser.REJECT)

    def test_prompt_strips_underscore_in_accept(self):
        self.assert_prompt_contains_text("accepT", accept="accep_T")

    def test_prompt_strips_underscore_in_reject(self):
        self.assert_prompt_contains_text("reJect", reject="re_Ject")

    def test_auto_accept_accepts_without_prompting(self):
        u = user.User(auto_accept=True)
        u._fileout = Mock(spec=sys.stderr)
        self.assertTrue(
            u.prompt(TestUser.TITLE, TestUser.MSG, TestUser.ACCEPT, TestUser.REJECT),
            "True expected!",
        )
        self.assertEqual(len(u._fileout.method_calls), 0, list(u._fileout.method_calls))

    def test_EOFError_in_prompt_caught_as_False(self):
        self.user._input.configure_mock(
            side_effect=EOFError, return_value=TestUser.REJECT
        )
        self.assertFalse(
            self.user.prompt(
                TestUser.TITLE, TestUser.MSG, TestUser.ACCEPT, TestUser.REJECT
            ),
            "False expected!",
        )
        self.user._input.assert_called_once_with()


class TestUser_quiet(unittest.TestCase):
    def setUp(self):
        self.user = user.User(quiet=True)
        self.user._fileout = Mock(spec=sys.stderr)

    def test_progress_can_begin_step_end(self):
        self.user.begin_progress("Foo", "Bar", 0)
        for i in range(10):
            self.user.step_progress()
        self.user.end_progress()

    def tearDown(self):
        self.assertEqual(
            len(self.user._fileout.method_calls),
            0,
            list(self.user._fileout.method_calls),
        )


class TestUser_progress(unittest.TestCase):
    def setUp(self):
        self.user = user.User()
        self.user._fileout = Mock(spec=sys.stderr)

    def test_can_step_using_with(self):
        # Collect baseline output from the old-style interface (begin/step/end)
        self._progress_begin_step_end()
        self.expected_output = list(self.user._fileout.method_calls)
        self.user._fileout.reset_mock()
        self.assertEqual(
            len(self.user._fileout.method_calls),
            0,
            list(self.user._fileout.method_calls),
        )

        with self.user.progress("Foo", "Bar", 0) as step:
            for i in range(10):
                step()

        # Output using `with' differs from one with `progress_...'
        self.assertEqual(self.expected_output, list(self.user._fileout.method_calls))

    def test_ends_progress_upon_exception_in_with(self):
        with patch("gramps.cli.user.User.end_progress") as MockEP:
            try:
                with self.user.progress("Foo", "Bar", 0) as step:
                    raise Exception()
            except Exception:
                pass
        self.assertTrue(MockEP.called)

    def _progress_begin_step_end(self):
        self.user.begin_progress("Foo", "Bar", 0)
        for i in range(10):
            self.user.step_progress()
        self.user.end_progress()

    def test_progress_percent(self):
        self.user.begin_progress("Foo", "Bar", 20)
        self.user._fileout.write.assert_called_with("Bar")
        for i in range(20):
            self.user.step_progress()
            pct = (i + 1) * 5
            self.user._fileout.write.assert_called_with("\r{:3d}% ".format(pct))
        self.user.end_progress()
        self.user._fileout.write.assert_called_with("\n")

    def test_progress_spinner(self):
        self.user.begin_progress("Foo", "Bar", 0)
        self.user._fileout.write.assert_called_with("Bar")
        for i in range(5):
            self.user.step_progress()
            spn = user._SPINNER[(i + 1) % 4]
            self.user._fileout.write.assert_called_with("\r  {}  ".format(spn))
        self.user.end_progress()
        self.user._fileout.write.assert_called_with("\n")


if __name__ == "__main__":
    unittest.main()



#! /usr/bin/env python3
"""
Gramps - a GTK+/GNOME based genealogy program

Copyright (c) 2016 Gramps Development Team

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
"""

import unittest
import os
import difflib
from unittest.mock import patch
from time import localtime, strptime
import tempfile

from gramps.test.test_util import Gramps
from gramps.gen.const import DATA_DIR
from gramps.gen.datehandler import set_format
from gramps.gen.user import User
from gramps.gen.utils.config import config

TREE_NAME = "Test_exporttest"
TEST_DIR = os.path.abspath(os.path.join(DATA_DIR, "tests"))


def mock_localtime(*args):
    """
    Mock up a dummy to replace the varying 'time string results'
    """
    return strptime("25 Dec 1999", "%d %b %Y")


def call(*args):
    """Call Gramps to perform the action with out and err captured"""
    # print("call:", args)
    gramps = Gramps(user=User())
    out, err = gramps.run(*args)
    # print("out:", out, "err:", err)
    return out, err


def do_it(srcfile, tstfile, dfilter=None):
    """based on tstfile, prepare an result export and compare with
    expected.
    """
    tst_file = os.path.join(TEST_DIR, srcfile)
    expect_file = os.path.join(TEST_DIR, tstfile)
    with tempfile.TemporaryDirectory() as tmpdirname:
        result_file = os.path.join(tmpdirname, tstfile)
        err = call(
            "-C", TREE_NAME, "-q", "--import", tst_file, "--export", result_file
        )[1]
        if "Cleaning up." not in err:
            return "Export failed, no 'Cleaning up.'"
        msg = compare(expect_file, result_file, dfilter)
        if not msg:
            # we will leave the result_file in place if there was an error.
            try:
                os.remove(result_file)
            except OSError:
                pass
            return
        else:
            return msg


def compare(expect_file, result_file, dfilter=None):
    """This uses the diff library to compare two files"""
    with open(
        expect_file, encoding="utf-8_sig", errors="surrogateescape"
    ) as exp_f, open(
        result_file, encoding="utf-8_sig", errors="surrogateescape"
    ) as res_f:
        diff = difflib.unified_diff(
            exp_f.readlines(), res_f.readlines(), n=2, lineterm="\n"
        )
        msg = ""
        fail = False
        for line in diff:
            if line == "--- \n" or line == "+++ \n":
                continue
            msg += line
            if dfilter:
                fail += dfilter(line)
            else:
                fail = True
        return msg if fail else ""


def gedfilt(line):
    """A filter for Gedcom files.
    This implements a filter that allows some differences to be ignored.
    The differences are not functional, but are related to changes in Gramps
    version, file date/time and filename.
    """

    def get_prev_token(back):
        if back > gedfilt.indx:
            return None
        return gedfilt.prev[gedfilt.indx - back][0]

    # pylint: disable=unsubscriptable-object
    if line.startswith("@@"):
        gedfilt.prev = [None] * 16
        gedfilt.indx = 0
        return False
    retval = True
    diftyp = line[0]
    line = line[1:].partition(" ")
    level = int(line[0])
    token, toss, line = line[2].partition(" ")
    if diftyp == " ":
        # save the line for later if needed to figure out the data element
        gedfilt.prev[gedfilt.indx] = token, level, line
        gedfilt.indx = (gedfilt.indx + 1) % 16
        retval = False
    elif diftyp == "-":
        # save the line for later if needed to figure out the data element
        gedfilt.prev[gedfilt.indx] = token, level, line
        gedfilt.indx = (gedfilt.indx + 1) % 16
        if token == "VERS" and get_prev_token(2) == "SOUR":
            # we must have a header with Gramps version
            retval = False
        elif token == "DATE" and get_prev_token(2) == "NAME":
            # we must have a header with file date
            retval = False
        elif token == "DATE" and get_prev_token(2) == "CHAN\n":
            # probably have a timestamp of last change
            retval = False
        elif token == "TIME" and get_prev_token(2) == "DATE":
            # probably have a header with file time
            retval = False
        elif token == "TIME" and get_prev_token(2) == "CHAN\n":
            # probably have a timestamp of last change
            retval = False
        elif token == "FILE" and line.endswith(".ged\n"):
            # probably have a header with file name
            retval = False
        elif token == "FILE" and "tests" in line:
            # probably have a media with file name
            retval = False
        elif token == "COPR" and "Copyright (c) " in line:
            # probably have a copyright line with year
            retval = False
    else:  # this is an addition
        if token == "VERS" and get_prev_token(1) == "VERS":
            # we must have a header with Gramps version
            retval = False
        elif token == "DATE" and (
            get_prev_token(2) == "NAME" or get_prev_token(3) == "NAME"
        ):
            # we must have a header with file date
            retval = False
        elif token == "DATE" and (
            get_prev_token(3) == "CHAN\n" or get_prev_token(4) == "CHAN\n"
        ):
            # probably have a timestamp of last change
            retval = False
        elif token == "TIME" and (
            get_prev_token(2) == "DATE" or get_prev_token(3) == "DATE"
        ):
            # probably have a header with file time
            retval = False
        elif token == "FILE" and line.endswith(".ged\n"):
            # probably have a header with file name
            retval = False
        elif token == "FILE" and "tests" in line:
            # probably have a media with file name
            retval = False
        elif token == "COPR" and "Copyright (c) " in line:
            # probably have a copyright line with year
            retval = False
    return retval


def vcffilt(line):
    """A filter for VCard files.
    This implements a filter that allows some differences to be ignored.
    The differences are not functional, but are related to changes in Gramps
    version.
    """
    diftyp = line[0]
    if diftyp == "@":
        retval = False
    elif diftyp == " ":
        retval = False
    elif "PRODID:" in line:  # Gramps version is on these lines
        retval = False
    else:
        retval = True
    return retval


class ExportControl(unittest.TestCase):
    """These tests compare various exported files with expected files,
    based on the matching '.gramps' test file as a source.
    As more types of exports are tested, we will need to provide some
    filters for the differences; some types of exports have Gramps versions,
    export dates, file names etc. that don't count as differences.
    """

    def setUp(self):
        config.set("behavior.date-before-range", 50)
        config.set("behavior.date-after-range", 50)
        config.set("behavior.date-about-range", 10)
        self.tearDown()  # removes it if it existed

    #        out, err = self.call("-C", TREE_NAME,
    #                            "--import", example)

    def tearDown(self):
        call("-y", "-q", "--remove", TREE_NAME)

    def test_csv(self):
        """Run a csv export test"""
        set_format(0)  # Use ISO date for test
        config.set("database.backend", "sqlite")
        src_file = "exp_sample_csv.gramps"
        tst_file = "exp_sample_csv.csv"
        msg = do_it(src_file, tst_file)
        if msg:
            self.fail(tst_file + ": " + msg)

    def test_ged(self):
        """Run a Gedcom export test"""
        config.set("preferences.place-auto", True)
        config.set("database.backend", "sqlite")
        src_file = "exp_sample.gramps"
        tst_file = "exp_sample_ged.ged"
        msg = do_it(src_file, tst_file, gedfilt)
        if msg:
            self.fail(tst_file + ": " + msg)

    def test_vcard(self):
        """Run a vcard export test"""
        config.set("preferences.place-auto", True)
        config.set("database.backend", "sqlite")
        src_file = "exp_sample.gramps"
        tst_file = "exp_sample.vcf"
        msg = do_it(src_file, tst_file, vcffilt)
        if msg:
            self.fail(tst_file + ": " + msg)

    @patch("gramps.plugins.export.exportvcalendar.time.localtime", mock_localtime)
    def test_vcs(self):
        """Run a Vcalandar export test"""
        config.set("preferences.place-auto", True)
        config.set("database.backend", "sqlite")
        src_file = "exp_sample.gramps"
        tst_file = "exp_sample.vcs"
        msg = do_it(src_file, tst_file)
        if msg:
            self.fail(tst_file + ": " + msg)

    def test_gw(self):
        """Run a Geneweb export test"""
        config.set("preferences.place-auto", True)
        config.set("database.backend", "sqlite")
        src_file = "exp_sample.gramps"
        tst_file = "exp_sample.gw"
        msg = do_it(src_file, tst_file)
        if msg:
            self.fail(tst_file + ": " + msg)

    def test_wft(self):
        """Run a Web Family Tree export test"""
        set_format(0)  # Use ISO date for test
        config.set("preferences.place-auto", True)
        config.set("database.backend", "sqlite")
        src_file = "exp_sample.gramps"
        tst_file = "exp_sample.wft"
        msg = do_it(src_file, tst_file)
        if msg:
            self.fail(tst_file + ": " + msg)


if __name__ == "__main__":
    unittest.main()



# Gramps - a GTK+/GNOME based genealogy program
#
# Copyright (C) 2011 Nick Hall
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
#

# -------------------------------------------------------------------------
#
# Gtk
#
# -------------------------------------------------------------------------
from gi.repository import Gtk
from gi.repository import Pango
from gi.repository.GLib import markup_escape_text

# -------------------------------------------------------------------------
#
# Gramps modules
#
# -------------------------------------------------------------------------
from gramps.gen.lib import UrlType
from gramps.gen.plug import Gramplet
from gramps.gen.const import COLON, GRAMPS_LOCALE as glocale

_ = glocale.translation.gettext


class RepositoryDetails(Gramplet):
    """
    Displays details for a repository.
    """

    def init(self):
        self.gui.WIDGET = self.build_gui()
        self.gui.get_container_widget().remove(self.gui.textview)
        self.gui.get_container_widget().add(self.gui.WIDGET)

    def build_gui(self):
        """
        Build the GUI interface.
        """
        self.top = Gtk.Box()
        vbox = Gtk.Box(orientation=Gtk.Orientation.VERTICAL)
        self.name = Gtk.Label(halign=Gtk.Align.START)
        self.name.set_selectable(True)
        vbox.pack_start(self.name, fill=True, expand=False, padding=7)
        self.grid = Gtk.Grid(orientation=Gtk.Orientation.VERTICAL)
        self.grid.set_column_spacing(10)
        vbox.pack_start(self.grid, fill=True, expand=False, padding=0)
        self.top.pack_start(vbox, fill=True, expand=False, padding=10)
        self.top.show_all()
        return self.top

    def add_row(self, title, value):
        """
        Add a row to the table.
        """
        label = Gtk.Label(
            label=title + COLON, halign=Gtk.Align.END, valign=Gtk.Align.START
        )
        label.set_selectable(True)
        label.show()
        value = Gtk.Label(label=value, halign=Gtk.Align.START)
        value.set_selectable(True)
        value.show()
        self.grid.add(label)
        self.grid.attach_next_to(value, label, Gtk.PositionType.RIGHT, 1, 1)

    def clear_grid(self):
        """
        Remove all the rows from the grid.
        """
        list(map(self.grid.remove, self.grid.get_children()))

    def db_changed(self):
        self.connect(self.dbstate.db, "repository-update", self.update)
        self.connect_signal("Repository", self.update)

    def update_has_data(self):
        active_handle = self.get_active("Repository")
        if active_handle:
            repo = self.dbstate.db.get_repository_from_handle(active_handle)
            self.set_has_data(repo is not None)
        else:
            self.set_has_data(False)

    def main(self):
        self.display_empty()
        active_handle = self.get_active("Repository")
        if active_handle:
            repo = self.dbstate.db.get_repository_from_handle(active_handle)
            self.top.hide()
            if repo:
                self.display_repo(repo)
                self.set_has_data(True)
            else:
                self.set_has_data(False)
            self.top.show()
        else:
            self.set_has_data(False)

    def display_repo(self, repo):
        """
        Display details of the active repository.
        """
        self.name.set_markup(
            "<span size='large' weight='bold'>%s</span>"
            % markup_escape_text(repo.get_name(), -1)
        )

        self.clear_grid()
        address_list = repo.get_address_list()
        if len(address_list) > 0:
            self.display_address(address_list[0])
            self.display_separator()
            phone = address_list[0].get_phone()
            if phone:
                self.add_row(_("Phone"), phone)

        self.display_url(repo, UrlType(UrlType.EMAIL))
        self.display_url(repo, UrlType(UrlType.WEB_HOME))
        self.display_url(repo, UrlType(UrlType.WEB_SEARCH))
        self.display_url(repo, UrlType(UrlType.WEB_FTP))

    def display_address(self, address):
        """
        Display an address.
        """
        lines = [line for line in address.get_text_data_list()[:-1] if line]
        self.add_row(_("Address"), "\n".join(lines))

    def display_url(self, repo, url_type):
        """
        Display an url of the given url type.
        """
        for url in repo.get_url_list():
            if url.get_type() == url_type:
                self.add_row(str(url_type), url.get_path())

    def display_empty(self):
        """
        Display empty details when no repository is selected.
        """
        self.name.set_text("")
        self.clear_grid()

    def display_separator(self):
        """
        Display an empty row to separate groupd of entries.
        """
        label = Gtk.Label()
        label.set_markup("<span font='sans 4'> </span>")
        label.set_selectable(True)
        label.show()
        self.grid.add(label)



#
# Gramps - a GTK+/GNOME based genealogy program
#
# Copyright (C) 2002-2006  Donald N. Allingham
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
#

"""
Package providing filtering framework for Gramps.
"""

# -------------------------------------------------------------------------
#
# GTK
#
# -------------------------------------------------------------------------
from gi.repository import Gtk
from gi.repository import GObject

# -------------------------------------------------------------------------
#
# Gramps modules
#
# -------------------------------------------------------------------------
from gramps.gen.filters import CustomFilters


# -------------------------------------------------------------------------
#
# FilterComboBox
#
# -------------------------------------------------------------------------
class FilterComboBox(Gtk.ComboBox):
    def set(self, local_filters, default=""):
        self.store = Gtk.ListStore(GObject.TYPE_STRING)
        self.set_model(self.store)
        cell = Gtk.CellRendererText()
        self.pack_start(cell, True)
        self.add_attribute(cell, "text", 0)

        self.map = {}

        active = 0
        cnt = 0
        for filt in local_filters:
            self.store.append(row=[filt.get_name()])
            self.map[str(filt.get_name())] = filt
            if default != "" and default == filt.get_name():
                active = cnt
            cnt += 1

        # for filt in SystemFilters.get_filters():
        # self.store.append(row=[filt.get_name()])
        # self.map[unicode(filt.get_name())] = filt
        # if default != "" and default == filt.get_name():
        # active = cnt
        # cnt += 1

        for filt in CustomFilters.get_filters():
            self.store.append(row=[filt.get_name()])
            self.map[str(filt.get_name())] = filt
            if default != "" and default == filt.get_name():
                active = cnt
            cnt += 1

        if active:
            self.set_active(active)
        elif len(local_filters):
            self.set_active(2)
        # elif len(SystemFilters.get_filters()):
        # self.set_active(4 + len(local_filters))
        elif len(CustomFilters.get_filters()):
            self.set_active(4 + len(local_filters))
        else:
            self.set_active(0)

    def get_value(self):
        active = self.get_active()
        if active < 0:
            return None
        key = str(self.store[active][0])
        return self.map[key]



#! /usr/bin/env python3
#
# Gramps - a GTK+/GNOME based genealogy program
#
# Copyright (c) 2016 Gramps Development Team
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
#

import unittest
import os
import shutil

from gramps.test.test_util import Gramps
from gramps.gen.user import User
from gramps.gen.const import DATA_DIR

# ddir = os.path.dirname(__file__)
# example = os.path.join(ddir, "..", "..", "..",
#                        "example", "gramps", "data.gramps")
# sample = os.path.join(ddir, "..", "..", "..",
#                       "example", "gedcom", "sample.ged")
example = os.path.join(DATA_DIR, "tests", "data.gramps")

TREE_NAME = "Test_reporttest"


class ReportControl:
    def tearDown(self):
        out, err = self.call("-y", "--remove", TREE_NAME)
        # out, err = self.call("-y", "--remove", TREE_NAME + "_import_gedcom")

    def call(self, *args):
        # print("call:", args)
        self.gramps = Gramps(user=User())
        out, err = self.gramps.run(*args)
        # print("out:", out, "err:", err)
        return out, err

    def __init__(self):
        super().__init__()
        self.tearDown()  # removes it if it existed
        out, err = self.call("-C", TREE_NAME, "--import", example)

    def addreport(self, class_, report_name, test_function, files, **options):
        test_name = "test_" + report_name.replace("-", "_")
        setattr(
            class_,
            test_name,
            dynamic_report_method(
                report_name + " " + TREE_NAME,
                test_function,
                files,
                "--force",
                "-O",
                TREE_NAME,
                "--action",
                "report",
                "--options",
                "name=%s" % report_name,
                **options,
            ),
        )

    def addcli(self, class_, report_name, test_function, files, *args, **options):
        test_name = "test_" + report_name.replace("-", "_")
        setattr(
            class_,
            test_name,
            dynamic_cli_method(report_name, test_function, files, *args, **options),
        )


def dynamic_report_method(report_name, test_function, files, *args, **options):
    args = list(args)
    args[-1] += "," + (",".join(["%s=%s" % (k, v) for (k, v) in options.items()]))
    options["files"] = files

    def test_method(self):  # This needs to have "test" in name
        out, err = self.call(*args)
        self.assertTrue(test_function(out, err, report_name, **options))

    return test_method


def dynamic_cli_method(report_name, test_function, files, *args, **options):
    options["files"] = files

    def test_method(self):  # This needs to have "test" in name
        out, err = self.call(*args)
        self.assertTrue(test_function(out, err, report_name, **options))

    return test_method


class TestDynamic(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        try:
            os.makedirs("temp")
        except:
            pass

    @classmethod
    def call(cls, *args):
        # print("call:", args)
        gramps = Gramps(user=User())
        out, err = gramps.run(*args)
        # print("out:", out, "err:", err)
        return out, err

    @classmethod
    def tearDownClass(cls):
        out, err = cls.call("-y", "--remove", TREE_NAME)
        # out, err = cls.call("-y", "--remove", TREE_NAME + "_import_gedcom")


reports = ReportControl()


def report_contains(text):
    def test_output_file(out, err, report_name, **options):
        ext = options["off"]
        with open(report_name + "." + ext) as fp:
            contents = fp.read()
        # print(contents)
        if options.get("files", []):
            for filename in options.get("files", []):
                if filename is None:
                    pass
                elif os.path.isdir(filename):
                    shutil.rmtree(filename)
                elif os.path.isfile(filename):
                    os.remove(filename)
                else:
                    raise Exception("can't find '%s' in order to delete it" % filename)
        elif os.path.isfile(report_name + "." + ext):
            os.remove(report_name + "." + ext)
        else:
            raise Exception(
                "can't find '%s' in order to delete it" % (report_name + "." + ext)
            )
        return text in contents

    return test_output_file


def err_does_not_contain(text):
    def test_output_file(dummy, err, report_name, **options):
        if options.get("files", []):
            for filename in options.get("files", []):
                if filename is None:
                    pass
                elif os.path.isdir(filename):
                    shutil.rmtree(filename)
                elif os.path.isfile(filename):
                    os.remove(filename)
                else:
                    raise Exception("can't find '%s' in order to delete it" % filename)
        else:
            ext = options["off"]
            if os.path.isfile(report_name + "." + ext):
                os.remove(report_name + "." + ext)
            else:
                raise Exception(
                    "can't find '%s' in order to delete it" % (report_name + "." + ext)
                )
        return text not in err

    return test_output_file


def err_does_contain(text):
    def test_output_file(dummy, err, report_name, **options):
        if options.get("files", []):
            for filename in options.get("files", []):
                if filename is None:
                    pass
                elif os.path.isdir(filename):
                    shutil.rmtree(filename)
                elif os.path.isfile(filename):
                    os.remove(filename)
                else:
                    raise Exception("can't find '%s' in order to delete it" % filename)
        else:
            ext = options["off"]
            if os.path.isfile(report_name + "." + ext):
                os.remove(report_name + "." + ext)
            else:
                raise Exception(
                    "can't find '%s' in order to delete it" % (report_name + "." + ext)
                )
        if isinstance(text, list):
            return all([(line in err) for line in text])
        else:
            return text in err

    return test_output_file


def out_does_contain(text):
    def test_output_file(out, dummy, report_name, **options):
        if options.get("files", []):
            for filename in options.get("files", []):
                if filename is None:
                    pass
                elif os.path.isdir(filename):
                    shutil.rmtree(filename)
                elif os.path.isfile(filename):
                    os.remove(filename)
                else:
                    raise Exception("can't find '%s' in order to delete it" % filename)
        else:
            ext = options["off"]
            if os.path.isfile(report_name + "." + ext):
                os.remove(report_name + "." + ext)
            else:
                raise Exception(
                    "can't find '%s' in order to delete it" % (report_name + "." + ext)
                )
        if isinstance(text, list):
            return all([(line in out) for line in text])
        else:
            return text in out

    return test_output_file


reports.addreport(
    TestDynamic,
    "tag_report",
    report_contains("I0037  Smith, Edwin Michael"),
    [],
    off="txt",
    tag="tag1",
)

reports.addreport(
    TestDynamic,
    "navwebpage",
    err_does_not_contain("Failed to write report."),
    ["/tmp/NAVWEB"],
    off="html",
    target="/tmp/NAVWEB",
)

reports.addreport(
    TestDynamic,
    "WebCal",
    err_does_not_contain("Failed to write report."),
    ["/tmp/WEBCAL"],
    off="html",
    target="/tmp/WEBCAL",
)

# THIRD-PARTY
# reports.addreport(TestDynamic, "database-differences-report",
#                  err_does_not_contain("Failed to write report."),
#                  [],
#                  off="txt", filename=example)

reports.addcli(
    TestDynamic,
    "export_gedcom",
    err_does_contain("Cleaning up."),
    ["test_export.ged"],
    "--force",
    "-O",
    TREE_NAME,
    "--export",
    "test_export.ged",
)

# reports.addcli(TestDynamic, "export_csv",
#                err_does_contain("Cleaning up."),
#                ["test_export.csv"],
#                "--force",
#                "-O", TREE_NAME,
#                "--export", "test_export.csv")

# reports.addcli(TestDynamic, "export_wtf",
#                err_does_contain("Cleaning up."),
#                ["test_export.wtf"],
#                "--force",
#                "-O", TREE_NAME,
#                "--export", "test_export.wtf")

reports.addcli(
    TestDynamic,
    "export_gw",
    err_does_contain("Cleaning up."),
    ["test_export.gw"],
    "--force",
    "-O",
    TREE_NAME,
    "--export",
    "test_export.gw",
)

reports.addcli(
    TestDynamic,
    "export_gpkg",
    err_does_contain("Cleaning up."),
    ["test_export.gpkg"],
    "--force",
    "-O",
    TREE_NAME,
    "--export",
    "test_export.gpkg",
)

reports.addcli(
    TestDynamic,
    "export_vcs",
    err_does_contain("Cleaning up."),
    ["test_export.vcs"],
    "--force",
    "-O",
    TREE_NAME,
    "--export",
    "test_export.vcs",
)

reports.addcli(
    TestDynamic,
    "export_vcf",
    err_does_contain("Cleaning up."),
    ["test_export.vcf"],
    "--force",
    "-O",
    TREE_NAME,
    "--export",
    "test_export.vcf",
)

report_list = [
    (
        "ancestor_chart",
        "svg",
        [
            "ancestor_chart %s.svg" % TREE_NAME,
            "ancestor_chart %s-2.svg" % TREE_NAME,
            "ancestor_chart %s-3.svg" % TREE_NAME,
            "ancestor_chart %s-4.svg" % TREE_NAME,
            "ancestor_chart %s-5.svg" % TREE_NAME,
            "ancestor_chart %s-6.svg" % TREE_NAME,
        ],
    ),  # Ancestor Tree
    ("ancestor_report", "txt", []),  # Ahnentafel Report
    ("birthday_report", "txt", []),  # Birthday and Anniversary Report
    (
        "calendar",
        "svg",
        [
            "calendar %s-10.svg" % TREE_NAME,
            "calendar %s-11.svg" % TREE_NAME,
            "calendar %s-12.svg" % TREE_NAME,
            "calendar %s-2.svg" % TREE_NAME,
            "calendar %s-3.svg" % TREE_NAME,
            "calendar %s-4.svg" % TREE_NAME,
            "calendar %s-5.svg" % TREE_NAME,
            "calendar %s-6.svg" % TREE_NAME,
            "calendar %s-7.svg" % TREE_NAME,
            "calendar %s-8.svg" % TREE_NAME,
            "calendar %s-9.svg" % TREE_NAME,
            "calendar %s.svg" % TREE_NAME,
        ],
    ),  # Calendar
    ("descend_chart", "svg", []),  # Descendant Tree
    ("descend_report", "txt", []),  # Descendant Report
    ("det_ancestor_report", "txt", []),  # Detailed Ancestral Report
    ("det_descendant_report", "txt", []),  # Detailed Descendant Report
    ("endofline_report", "txt", []),  # End of Line Report
    ("family_descend_chart", "svg", []),  # Family Descendant Tree
    ("family_group", "txt", []),  # Family Group Report
    # COULD be dot ("familylines_graph", "dot", []),  # Family Lines Graph
    ("fan_chart", "svg", []),  # Fan Chart
    # COULD be dot ("hourglass_graph", "dot", []),  # Hourglass Graph
    ("indiv_complete", "txt", []),  # Complete Individual Report
    ("kinship_report", "txt", []),  # Kinship Report
    ("notelinkreport", "txt", []),  # Note Link Report
    ("number_of_ancestors", "txt", []),  # Number of Ancestors Report
    # NEED a place ("place_report", "txt", []),  # Place Report
    ("records", "txt", []),  # Records Report
    # COULD be dot ("rel_graph", "dot", []),  # Relationship Graph
    (
        "statistics_chart",
        "svg",
        [
            "statistics_chart %s.svg" % TREE_NAME,  # Statistics Charts
            "statistics_chart %s-2.svg" % TREE_NAME,
            "statistics_chart %s-3.svg" % TREE_NAME,
        ],
    ),
    ("summary", "txt", []),  # Database Summary Report
    (
        "timeline",
        "svg",
        [
            "timeline %s.svg" % TREE_NAME,
            "timeline %s-2.svg" % TREE_NAME,
        ],
    ),  # Timeline Chart
]

for _report_name, _off, _files in report_list:
    reports.addreport(
        TestDynamic,
        _report_name,
        err_does_not_contain("Failed to write report."),
        files=_files,
        off=_off,
    )

txt_list = [
    "W: Early marriage, Family: F0000, Smith, Martin and Jefferson, Elna",
    "W: Multiple parents, Person: I0061, Jones, Roberta Michele",
    "W: Multiple parents, Person: I0063, Jones, Frank Albert",
    "W: Multiple parents, Person: I0076, Smith, Marie Astri",
    "W: Multiple parents, Person: I0077, Smith, Susan Elizabeth",
    "W: Old age but no death, Person: I0004, Smith, Ingeman",
    "W: Old age but no death, Person: I0009, Smith, Emil",
    "W: Old age but no death, Person: I0011, Smith, Hanna",
    "W: Old age but no death, Person: I0058, Smith, Elaine Marie",
    "W: Old age but no death, Person: I0072, Iverson, Alice Hannah",
]
reports.addcli(
    TestDynamic,
    "tool_verify",
    out_does_contain(txt_list),
    [None],
    "--force",
    "-O",
    TREE_NAME,
    "-y",
    "--action",
    "tool",
    "--options",
    "name=verify",
)

txt_list = [
    "6 media objects were referenced, but not found",
    "References to 6 missing media objects were kept",
]
reports.addcli(
    TestDynamic,
    "tool_check",
    out_does_contain(txt_list),
    [None],
    "--force",
    "-O",
    TREE_NAME,
    "-y",
    "--action",
    "tool",
    "--options",
    "name=check",
)

if __name__ == "__main__":
    unittest.main()



# -*- coding: utf-8 -*-
#
# Gramps - a GTK+/GNOME based genealogy program
#
# Copyright (C) 2004-2006  Donald N. Allingham
# Copyright (C) 2015       Lajos Nemeséri <nemeseril@gmail.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
#

"""
Hungarian-specific classes for parsing and displaying dates.
"""

# -------------------------------------------------------------------------
#
# Python modules
#
# -------------------------------------------------------------------------
import re

# -------------------------------------------------------------------------
#
# Gramps modules
#
# -------------------------------------------------------------------------
from ..lib.date import Date
from ._dateparser import DateParser
from ._datedisplay import DateDisplay
from ._datehandler import register_datehandler


# -------------------------------------------------------------------------
#
# Hungarian parser
#
#
# -------------------------------------------------------------------------
class DateParserHU(DateParser):
    month_to_int = DateParser.month_to_int

    month_to_int["-"] = 0  # to make the Zero month to work

    month_to_int["január"] = 1
    month_to_int["jan"] = 1
    month_to_int["jan."] = 1
    month_to_int["I"] = 1
    #    month_to_int["i"] = 1

    month_to_int["február"] = 2
    month_to_int["feb"] = 2
    month_to_int["feb."] = 2
    month_to_int["II"] = 2
    #    month_to_int["ii"] = 2

    month_to_int["március"] = 3
    month_to_int["márc"] = 3
    month_to_int["márc."] = 3
    month_to_int["III"] = 3
    #    month_to_int["iii"] = 3

    month_to_int["április"] = 4
    month_to_int["ápr"] = 4
    month_to_int["ápr."] = 4
    month_to_int["IV"] = 4
    #    month_to_int["iv"] = 4

    month_to_int["május"] = 5
    month_to_int["máj"] = 5
    month_to_int["máj."] = 5
    month_to_int["V"] = 5
    #    month_to_int["v"] = 5

    month_to_int["június"] = 6
    month_to_int["jún"] = 6
    month_to_int["jún."] = 6
    month_to_int["VI"] = 6
    #    month_to_int["vi"] = 6

    month_to_int["július"] = 7
    month_to_int["júl"] = 7
    month_to_int["júl."] = 7
    month_to_int["VII"] = 7
    #    month_to_int["vii"] = 7

    month_to_int["augusztus"] = 8
    month_to_int["aug"] = 8
    month_to_int["aug."] = 8
    month_to_int["VIII"] = 8
    #    month_to_int["viii"] = 8

    month_to_int["szeptember"] = 9
    month_to_int["szept"] = 9
    month_to_int["szept."] = 9
    month_to_int["IX"] = 9
    #    month_to_int["ix"] = 9

    month_to_int["október"] = 10
    month_to_int["okt"] = 10
    month_to_int["okt."] = 10
    month_to_int["X"] = 10
    #    month_to_int["x"] = 10

    month_to_int["november"] = 11
    month_to_int["nov"] = 11
    month_to_int["nov."] = 11
    month_to_int["XI"] = 11
    #    month_to_int["xi"] = 11

    month_to_int["december"] = 12
    month_to_int["dec"] = 12
    month_to_int["dec."] = 12
    month_to_int["XII"] = 12
    #    month_to_int["xii"] = 12

    # -----------------------------------------------------------------------
    #
    #  Alternative and latin names - not verified
    #
    # -----------------------------------------------------------------------
    # Other common latin names

    #    month_to_int["januaris"] = 01
    #    month_to_int["januarii"] = 01
    #    month_to_int["januarius"] = 01
    #    month_to_int["februaris"] = 02
    #    month_to_int["februarii"] = 02
    #    month_to_int["februarius"] = 02
    #    month_to_int["martii"] = 03
    #    month_to_int["martius"] = 03
    #    month_to_int["aprilis"] = 04
    #    month_to_int["maius"] = 05
    #    month_to_int["maii"] = 05
    #    month_to_int["junius"] = 06
    #    month_to_int["junii"] = 06
    #    month_to_int["julius"] = 07
    #    month_to_int["julii"] = 07
    #    month_to_int["augustus"] = 08
    #    month_to_int["augusti"] = 08
    #    month_to_int["septembris"] = 09
    #    month_to_int["7bris"] = 09
    #    month_to_int["september"] = 09
    #    month_to_int["october"] = 10
    #    month_to_int["octobris"] = 10
    #    month_to_int["8bris"] = 10
    #    month_to_int["novembris"] = 11
    #    month_to_int["9bris"] = 11
    #    month_to_int["november"] = 11
    #    month_to_int["decembris"] = 12
    #    month_to_int["10bris"] = 12
    #    month_to_int["xbris"] = 12
    #    month_to_int["december"] = 12

    # old Hungarian names

    #    month_to_int["Boldogasszony hava"] = 01
    #    month_to_int["Fergeteg hava"] = 01
    #    month_to_int["Böjtelő hava"] = 02
    #    month_to_int["Jégbontó hava"] = 02
    #    month_to_int["Böjtmás hava"] = 03
    #    month_to_int["Kikelet hava"] = 03
    #    month_to_int["Szent György hava"] = 04
    #    month_to_int["Szelek hava"] = 04
    #    month_to_int["Pünkösd hava"] = 05
    #    month_to_int["Ígéret hava"] = 05
    #    month_to_int["Szent Iván hava"] = 06
    #    month_to_int["Napisten hava"] = 06
    #    month_to_int["Szent Jakab hava"] = 07
    #    month_to_int["Áldás hava"] = 07
    #    month_to_int["Kisasszony hava"] = 08
    #    month_to_int["Újkenyér hava"] = 08
    #    month_to_int["Szent Mihály hava"] = 09
    #    month_to_int["Földanya hava"] = 09
    #    month_to_int["Mindszent hava"] = 10
    #    month_to_int["Magvető hava"] = 10
    #    month_to_int["Szent András hava"] = 11
    #    month_to_int["Enyészet hava"] = 11
    #    month_to_int["Karácsony hava"] = 12
    #    month_to_int["Álom hava"] = 12

    modifier_to_int = {
        "-tól": Date.MOD_FROM,
        "-ig": Date.MOD_TO,
    }
    modifier_after_to_int = {
        "előtt": Date.MOD_BEFORE,
        "körül": Date.MOD_ABOUT,
        "után": Date.MOD_AFTER,
    }

    quality_to_int = {
        "becsült": Date.QUAL_ESTIMATED,
        "hozzávetőleg": Date.QUAL_ESTIMATED,
        "becs.": Date.QUAL_ESTIMATED,
        "számított": Date.QUAL_CALCULATED,
        "körülbelül": Date.QUAL_ESTIMATED,
        "számolt": Date.QUAL_CALCULATED,
        "szám.": Date.QUAL_CALCULATED,
    }

    bce = [
        "időszámításunk előtt",
        "időszámítás előtt",
        "i. e.",
        "Krisztus előtt",
        "Krisztus előtti",
        "Kr. e.",
    ] + DateParser.bce

    calendar_to_int = {
        "Gergely": Date.CAL_GREGORIAN,
        "Julián": Date.CAL_JULIAN,
        "julián": Date.CAL_JULIAN,
        "héber": Date.CAL_HEBREW,
        "iszlám": Date.CAL_ISLAMIC,
        "francia köztársasági": Date.CAL_FRENCH,
        "perzsa": Date.CAL_PERSIAN,
        "svéd": Date.CAL_SWEDISH,
    }

    def init_strings(self):
        # Compiles regular expression strings for matching dates
        DateParser.init_strings(self)

        self._numeric = re.compile(r"((\d+)[/\.])?\s*((\d+)[/\.])?\s*(\d+)[/\. ]?$")
        # this next RE has the (possibly-slashed) year at the string's start
        self._text2 = re.compile(
            r"((\d+)(/\d+)?\.)?\s+?%s\.?\s*(\d+\.)?\s*$" % self._mon_str, re.IGNORECASE
        )
        _span_1 = [r"-tó\(ő\)l", "-tól", "-től"]
        _span_2 = ["-ig"]
        _range_1 = ["és"]
        _range_2 = ["között"]
        self._span = re.compile(
            r"(?P<start>.+)(%s)\s+(?P<stop>.+)(%s)"
            % ("|".join(_span_1), "|".join(_span_2)),
            re.IGNORECASE,
        )
        self._range = re.compile(
            r"(?P<start>.+)\s+(%s)\s+(?P<stop>.+)\s+(%s)"
            % ("|".join(_range_1), "|".join(_range_2)),
            re.IGNORECASE,
        )

    def _get_int(self, val):
        """
        Convert the string to an integer if the value is not None. If the
        value is None, a zero is returned
        """
        if val is None:
            return 0
        else:
            return int(val.replace(".", ""))


# -------------------------------------------------------------------------
#
# Hungarian display
#
# -------------------------------------------------------------------------
class DateDisplayHU(DateDisplay):
    """
    Hungarian language date display class.
    """

    _bce_str = "i. e. %s"

    roman_months = (
        "-.",
        "I.",
        "II.",
        "III.",
        "IV.",
        "V.",
        "VI.",
        "VII.",
        "VIII.",
        "IX.",
        "X.",
        "XI.",
        "XII.",
    )

    formats = (
        "ÉÉÉÉ-HH-NN (ISO)",  # 0
        "Alapértelmezett éééé. hh. nn.",  # 1
        "Év hónap nap",  # year, full month name, day     # 2
        "Év hó nap",  # year, short month name, day     # 3
        "Év római h.sz. nap",  # year, Roman number, day  # 4
    )
    # this definition must agree with its "_display_calendar" method

    display = DateDisplay.display_formatted

    def _display_calendar(self, date_val, long_months, short_months=None, inflect=""):
        # this must agree with its locale-specific "formats" definition

        year = self._slash_year(date_val[2], date_val[3])

        if short_months is None:
            # Let the short formats work the same as long formats
            short_months = long_months

        if self.format == 0:
            return self.display_iso(date_val)

        elif self.format == 1:
            # Base defined Hungarian form
            if date_val[3]:
                return self.display_iso(date_val)
            else:
                if date_val[0] == 0:  # No day
                    if date_val[1] == 0:  # No month -> year
                        value = "%s" % year
                    else:
                        value = "%s. %02d." % (
                            year,
                            date_val[1],
                        )  # If no day -> year, month
                else:
                    value = "%s. %02d. %02d." % (year, date_val[1], date_val[0])

        elif self.format == 2:
            # year, full month name, day

            if date_val[0] == 0:
                if date_val[1] == 0:
                    value = "%s" % year
                else:
                    value = "%s. %s" % (
                        year,
                        self.long_months[date_val[1]],
                    )  # If no day -> year, month
            else:
                if date_val[1] == 0:
                    value = "%s. %s %02d." % (
                        year,
                        "-",
                        date_val[0],
                    )  # To indicate somehow if the month is missing
                else:
                    value = "%s. %s %02d." % (
                        year,
                        self.long_months[date_val[1]],
                        date_val[0],
                    )

        elif self.format == 3:
            # year, short month name, day

            if date_val[0] == 0:
                if date_val[1] == 0:
                    value = "%s" % year
                else:
                    value = "%s. %s" % (
                        year,
                        self.short_months[date_val[1]],
                    )  # If no day -> year, month
            else:
                if date_val[1] == 0:
                    value = "%s. %s %02d." % (
                        year,
                        "-.",
                        date_val[0],
                    )  # To indicate somehow if the month is missing
                else:
                    value = "%s. %s %02d." % (
                        year,
                        self.short_months[date_val[1]],
                        date_val[0],
                    )

        elif self.format == 4:
            # year, Roman number, day

            if date_val[0] == 0:
                if date_val[1] == 0:
                    value = "%s" % year
                else:
                    value = "%s. %s" % (
                        year,
                        self.roman_months[date_val[1]],
                    )  # If no day -> year, month
            else:
                value = "%s. %s %02d." % (
                    year,
                    self.roman_months[date_val[1]],
                    date_val[0],
                )

        else:
            # day month_name year
            value = self.dd_dformat04(date_val, inflect, long_months)

        if date_val[2] < 0:
            # TODO fix BUG 7064: non-Gregorian calendars wrongly use BCE notation for negative dates
            return self._bce_str % value
        else:
            return value


# -------------------------------------------------------------------------
#
# Register classes
#
# -------------------------------------------------------------------------
register_datehandler(
    ("hu_HU", "hu", "hungarian", "Hungarian", "magyar", ("%Y-%m-%d",)),
    DateParserHU,
    DateDisplayHU,
)



#
# Gramps - a GTK+/GNOME based genealogy program
#
# Copyright (C) 2011       Helge Herz
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
#

# -------------------------------------------------------------------------
#
# Standard Python modules
#
# -------------------------------------------------------------------------
from ....const import GRAMPS_LOCALE as glocale

_ = glocale.translation.gettext

# -------------------------------------------------------------------------
#
# Gramps modules
#
# -------------------------------------------------------------------------
from .. import Rule


# -------------------------------------------------------------------------
#
# Typing modules
#
# -------------------------------------------------------------------------
from ....lib import Source
from ....db import Database


# -------------------------------------------------------------------------
# "Sources having a title that contain a substring"
# -------------------------------------------------------------------------
class MatchesTitleSubstringOf(Rule):
    """Source title containing <substring>"""

    labels = [_("Text:")]
    name = _("Sources with title containing <text>")
    description = _("Matches sources whose title contains a " "certain substring")
    category = _("General filters")
    allow_regex = True

    def apply_to_one(self, db: Database, source: Source) -> bool:
        """Apply the filter"""
        return self.match_substring(0, source.title)



#
# Gramps - a GTK+/GNOME based genealogy program
#
# Copyright (C) 2002-2006  Donald N. Allingham
# Copyright (C) 2011       Tim G L Lyons
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
#

# -------------------------------------------------------------------------
#
# Standard Python modules
#
# -------------------------------------------------------------------------
from ....const import GRAMPS_LOCALE as glocale

_ = glocale.translation.gettext

# -------------------------------------------------------------------------
#
# Gramps modules
#
# -------------------------------------------------------------------------
from .._regexpidbase import RegExpIdBase


# -------------------------------------------------------------------------
#
# Typing modules
#
# -------------------------------------------------------------------------
from ....lib import Citation
from ....db import Database


# -------------------------------------------------------------------------
#
# HasIdOf
#
# -------------------------------------------------------------------------
class RegExpSourceIdOf(RegExpIdBase):
    """
    Rule that checks for a citation  whose Gramps ID
    matches regular expression.
    """

    name = _("Citations with Source Id containing <text>")
    description = _(
        "Matches citations whose source has a Gramps ID that "
        "matches the regular expression"
    )
    category = _("Source filters")

    def apply_to_one(self, dbase: Database, citation: Citation) -> bool:  # type: ignore[override]
        source = dbase.get_source_from_handle(citation.source_handle)
        return RegExpIdBase.apply_to_one(self, dbase, source)



# -*- coding: utf-8 -*-
#
# Gramps - a GTK+/GNOME based genealogy program
#
# Copyright (C) 2013 Vassilii Khachaturov <vassilii@tarunz.org>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
#

"""Unittest for argparser.py"""

import unittest
from unittest.mock import Mock
from ..argparser import ArgParser


class TestArgParser(unittest.TestCase):
    def setUp(self):
        pass

    def create_parser(*self_and_args):
        return ArgParser(list(self_and_args))

    def triggers_option_error(self, option):
        ap = self.create_parser(option)
        return (str(ap.errors).find("option " + option) >= 0, ap)

    def test_wrong_argument_triggers_option_error(self):
        bad, ap = self.triggers_option_error("--I-am-a-wrong-argument")
        self.assertTrue(bad, ap.__dict__)

    def test_y_shortopt_sets_auto_accept(self):
        bad, ap = self.triggers_option_error("-y")

        self.assertFalse(bad)

        expected_errors = [
            (
                "Error parsing the arguments",
                "Error parsing the arguments: [ -y ] \n"
                + "To use in the command-line mode, supply at least one input file to process.",
            )
        ]
        self.assertEqual(expected_errors, ap.errors)

        self.assertTrue(ap.auto_accept)

    def test_yes_longopt_sets_auto_accept(self):
        bad, ap = self.triggers_option_error("--yes")
        self.assertFalse(bad, ap.errors)
        self.assertTrue(ap.auto_accept)

    def test_q_shortopt_sets_quiet(self):
        bad, ap = self.triggers_option_error("-q")
        self.assertFalse(bad, ap.errors)
        self.assertTrue(ap.quiet)

    def test_quiet_longopt_sets_quiet(self):
        bad, ap = self.triggers_option_error("--quiet")
        self.assertFalse(bad, ap.errors)
        self.assertTrue(ap.quiet)

    def test_quiet_exists_by_default(self):
        ap = self.create_parser()
        self.assertTrue(hasattr(ap, "quiet"))

    def test_auto_accept_unset_by_default(self):
        ap = self.create_parser()
        self.assertFalse(ap.auto_accept)

    def test_exception(self):
        argument_parser = self.create_parser("-O")

        expected_errors = [
            (
                "Error parsing the arguments",
                "option -O requires argument\n"
                "Error parsing the arguments: [ -O ] \n"
                "Type gramps --help for an overview of commands, or read the manual pages.",
            )
        ]
        self.assertEqual(expected_errors, argument_parser.errors)

    def test_option_with_multiple_arguments(self):
        argument_parser = self.create_parser("-l", "family_tree_name")
        self.assertEqual(argument_parser.database_names, ["family_tree_name"])


if __name__ == "__main__":
    unittest.main()



# -*- coding: utf-8 -*-
#
# Gramps - a GTK+/GNOME based genealogy program
#
# Copyright (C) 2004-2006  Donald N. Allingham
# Copyright (C) 2013-2014  Fedir Zinchuk <fedikw[at]gmail.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
#

"""
Ukrainian-specific classes for parsing and displaying dates.
"""

# -------------------------------------------------------------------------
#
# Python modules
#
# -------------------------------------------------------------------------
import re

# -------------------------------------------------------------------------
#
# Gramps modules
#
# -------------------------------------------------------------------------

from ..lib.date import Date
from ._dateparser import DateParser
from ._datedisplay import DateDisplay
from ._datehandler import register_datehandler


# -------------------------------------------------------------------------
#
# Ukrainian parser
#
# -------------------------------------------------------------------------
class DateParserUK(DateParser):
    """
    Convert a text string into a :class:`.Date` object. If the date cannot be
    converted, the text string is assigned.
    """

    # modifiers before the date
    modifier_to_int = {
        "перед": Date.MOD_BEFORE,
        "раніше": Date.MOD_BEFORE,
        "після": Date.MOD_AFTER,
        "п.": Date.MOD_AFTER,
        "за": Date.MOD_AFTER,
        "приблизно": Date.MOD_ABOUT,
        "прибл.": Date.MOD_ABOUT,
        "приб.": Date.MOD_ABOUT,
        "близько": Date.MOD_ABOUT,
        "бл.": Date.MOD_ABOUT,
        "біля": Date.MOD_ABOUT,
        "від": Date.MOD_FROM,
        "до": Date.MOD_TO,
    }

    quality_to_int = {
        "за оцінкою": Date.QUAL_ESTIMATED,
        "за оц.": Date.QUAL_ESTIMATED,
        "оцінено": Date.QUAL_ESTIMATED,
        "орієнтовно": Date.QUAL_ESTIMATED,
        "приблизно": Date.QUAL_ESTIMATED,
        "прибл.": Date.QUAL_ESTIMATED,
        "підраховано": Date.QUAL_CALCULATED,
        "підрах.": Date.QUAL_CALCULATED,
        "розраховано": Date.QUAL_CALCULATED,
        "розрахунково": Date.QUAL_CALCULATED,
        "розрах.": Date.QUAL_CALCULATED,
    }

    bce = [
        "до нашої ери",
        "до н. е.",
        "до н.е.",
        "до народження Христа",
    ] + DateParser.bce

    def init_strings(self):
        """
        This method compiles regular expression strings for matching dates.

        See DateParser.init_strings()
        """
        DateParser.init_strings(self)

        DateParser.hebrew_to_int.update(
            {
                "мархешван": 2,
                "ве адар": 7,
                "адар бет": 7,
                "авів": 8,
            }
        )

        _span_1 = ["з", "від"]
        # b.c.e. pattern also have "до" so skip "до н."
        _span_2 = ["по", r"до(?!\s+н)"]
        _range_1 = ["між"]
        _range_2 = ["і", "та"]
        self._span = re.compile(
            r"(%s)\s+(?P<start>.+)\s+(%s)\s+(?P<stop>.+)"
            % ("|".join(_span_1), "|".join(_span_2)),
            re.IGNORECASE,
        )
        self._range = re.compile(
            r"(%s)\s+(?P<start>.+)\s+(%s)\s+(?P<stop>.+)"
            % ("|".join(_range_1), "|".join(_range_2)),
            re.IGNORECASE,
        )


# -------------------------------------------------------------------------
#
# Ukrainian displayer
#
# -------------------------------------------------------------------------
class DateDisplayUK(DateDisplay):
    """
    Ukrainian language date display class.
    """

    _bce_str = "%s до н.е."

    display = DateDisplay.display_formatted


# -------------------------------------------------------------------------
#
# Register classes
#
# -------------------------------------------------------------------------
register_datehandler(
    ("uk_UA", "uk", "ukrainian", "Ukrainian", ("%d.%m.%Y",)),
    DateParserUK,
    DateDisplayUK,
)



# Gramps - a GTK+/GNOME based genealogy program
#
# Copyright (C) 2007-2009  Douglas S. Blank <doug.blank@gmail.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

# ------------------------------------------------------------------------
#
# Python modules
#
# ------------------------------------------------------------------------
from collections import defaultdict

# ------------------------------------------------------------------------
#
# Gramps modules
#
# ------------------------------------------------------------------------
from gramps.gen.plug import Gramplet
from gramps.gen.config import config
from gramps.gen.const import GRAMPS_LOCALE as glocale
from gramps.gen.plug.menu import NumberOption

_ = glocale.translation.sgettext

# ------------------------------------------------------------------------
#
# Constants
#
# ------------------------------------------------------------------------
_YIELD_INTERVAL = 350

NUM_SURNAMES = _("Number of Surnames to display")


# ------------------------------------------------------------------------
#
# TopSurnamesGramplet class
#
# ------------------------------------------------------------------------
class TopSurnamesGramplet(Gramplet):
    def init(self):
        self.set_tooltip(_("Double-click surname for details"))
        self.top_size = 10  # will be overwritten in load
        self.set_text(_("No Family Tree loaded."))

    def db_changed(self):
        self.connect(self.dbstate.db, "person-add", self.update)
        self.connect(self.dbstate.db, "person-delete", self.update)
        self.connect(self.dbstate.db, "person-update", self.update)
        self.connect(self.dbstate.db, "person-rebuild", self.update)
        self.connect(self.dbstate.db, "family-rebuild", self.update)
        self.set_text(_("No Family Tree loaded."))

    def build_options(self):
        self.add_option(NumberOption(NUM_SURNAMES, self.top_size, 10, 1000))

    def save_options(self):
        self.top_size = int(self.get_option(NUM_SURNAMES).get_value())

    def on_load(self):
        if len(self.gui.data) == 1:
            self.top_size = int(self.gui.data[0])

    def on_save(self):
        self.gui.data = [self.top_size]

    def main(self):
        self.set_text(_("Processing...") + "\n")
        surnames = defaultdict(int)
        representative_handle = {}

        cnt = 0
        for person in self.dbstate.db.iter_people():
            allnames = [person.get_primary_name()] + person.get_alternate_names()
            allnames = set([name.get_group_name().strip() for name in allnames])
            for surname in allnames:
                surnames[surname] += 1
                representative_handle[surname] = person.handle
            cnt += 1
            if not cnt % _YIELD_INTERVAL:
                yield True

        total_people = cnt
        surname_sort = []
        total = 0

        cnt = 0
        for surname in surnames:
            surname_sort.append((surnames[surname], surname))
            total += surnames[surname]
            cnt += 1
            if not cnt % _YIELD_INTERVAL:
                yield True

        total_surnames = cnt
        surname_sort.sort(reverse=True)
        line = 0
        ### All done!
        self.set_text("")
        nosurname = config.get("preferences.no-surname-text")
        for count, surname in surname_sort:
            text = "%s, " % (surname if surname else nosurname)
            text += "%d%% (%d)\n" % (int((float(count) / total) * 100), count)
            self.append_text(" %d. " % (line + 1))
            self.link(text, "Surname", representative_handle[surname])
            line += 1
            if line >= self.top_size:
                break
        self.append_text(
            ("\n" + _("Total unique surnames") + ": %d\n") % total_surnames
        )
        self.append_text((_("Total people") + ": %d") % total_people, "begin")



# -*- coding: iso-8859-1 -*-
#
# Gramps - a GTK+/GNOME based genealogy program
#
# Copyright (C) 2004-2005  Donald N. Allingham
# Copyright (C) 2012       Doug Blank <doug.blank@gmail.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
#

# -------------------------------------------------------------------------
#
# Python modules
#
# -------------------------------------------------------------------------
import locale

# -------------------------------------------------------------------------
#
# Gramps modules
#
# -------------------------------------------------------------------------
from ..const import GRAMPS_LOCALE as glocale  # TODO unneeded? (see codeset)

"""
Some OS environments do not support the locale.nl_langinfo() method
of determing month names and other date related information.

If nl_langinfo fails, that means we have to resort to shinanigans with
strftime.

Since these routines return values encoded into selected character
set, we have to convert to unicode.
"""
codeset = glocale.encoding  # TODO I don't think "codeset" is used anymore

try:
    # here only for the upgrade tool, see _datestrings.py __main__
    _deprecated_long_months = (
        "",
        locale.nl_langinfo(locale.MON_1),
        locale.nl_langinfo(locale.MON_2),
        locale.nl_langinfo(locale.MON_3),
        locale.nl_langinfo(locale.MON_4),
        locale.nl_langinfo(locale.MON_5),
        locale.nl_langinfo(locale.MON_6),
        locale.nl_langinfo(locale.MON_7),
        locale.nl_langinfo(locale.MON_8),
        locale.nl_langinfo(locale.MON_9),
        locale.nl_langinfo(locale.MON_10),
        locale.nl_langinfo(locale.MON_11),
        locale.nl_langinfo(locale.MON_12),
    )

    _deprecated_short_months = (
        "",
        locale.nl_langinfo(locale.ABMON_1),
        locale.nl_langinfo(locale.ABMON_2),
        locale.nl_langinfo(locale.ABMON_3),
        locale.nl_langinfo(locale.ABMON_4),
        locale.nl_langinfo(locale.ABMON_5),
        locale.nl_langinfo(locale.ABMON_6),
        locale.nl_langinfo(locale.ABMON_7),
        locale.nl_langinfo(locale.ABMON_8),
        locale.nl_langinfo(locale.ABMON_9),
        locale.nl_langinfo(locale.ABMON_10),
        locale.nl_langinfo(locale.ABMON_11),
        locale.nl_langinfo(locale.ABMON_12),
    )

    # Gramps day number: Sunday => 1, Monday => 2, etc
    # "Return name of the n-th day of the week. Warning:
    #  This follows the US convention of DAY_1 being Sunday,
    #  not the international convention (ISO 8601) that Monday
    #  is the first day of the week."
    # see http://docs.python.org/library/locale.html
    _deprecated_long_days = (
        "",
        locale.nl_langinfo(locale.DAY_1),  # Sunday
        locale.nl_langinfo(locale.DAY_2),  # Monday
        locale.nl_langinfo(locale.DAY_3),  # Tuesday
        locale.nl_langinfo(locale.DAY_4),  # Wednesday
        locale.nl_langinfo(locale.DAY_5),  # Thursday
        locale.nl_langinfo(locale.DAY_6),  # Friday
        locale.nl_langinfo(locale.DAY_7),  # Saturday
    )

    _deprecated_short_days = (
        "",
        locale.nl_langinfo(locale.ABDAY_1),  # Sun
        locale.nl_langinfo(locale.ABDAY_2),  # Mon
        locale.nl_langinfo(locale.ABDAY_3),  # Tue
        locale.nl_langinfo(locale.ABDAY_4),  # Wed
        locale.nl_langinfo(locale.ABDAY_5),  # Thu
        locale.nl_langinfo(locale.ABDAY_6),  # Fri
        locale.nl_langinfo(locale.ABDAY_7),  # Sat
    )

except:
    import time

    _deprecated_long_months = (
        "",
        time.strftime("%B", (1, 1, 1, 1, 1, 1, 1, 1, 1)),
        time.strftime("%B", (1, 2, 1, 1, 1, 1, 1, 1, 1)),
        time.strftime("%B", (1, 3, 1, 1, 1, 1, 1, 1, 1)),
        time.strftime("%B", (1, 4, 1, 1, 1, 1, 1, 1, 1)),
        time.strftime("%B", (1, 5, 1, 1, 1, 1, 1, 1, 1)),
        time.strftime("%B", (1, 6, 1, 1, 1, 1, 1, 1, 1)),
        time.strftime("%B", (1, 7, 1, 1, 1, 1, 1, 1, 1)),
        time.strftime("%B", (1, 8, 1, 1, 1, 1, 1, 1, 1)),
        time.strftime("%B", (1, 9, 1, 1, 1, 1, 1, 1, 1)),
        time.strftime("%B", (1, 10, 1, 1, 1, 1, 1, 1, 1)),
        time.strftime("%B", (1, 11, 1, 1, 1, 1, 1, 1, 1)),
        time.strftime("%B", (1, 12, 1, 1, 1, 1, 1, 1, 1)),
    )

    _deprecated_short_months = (
        "",
        time.strftime("%b", (1, 1, 1, 1, 1, 1, 1, 1, 1)),
        time.strftime("%b", (1, 2, 1, 1, 1, 1, 1, 1, 1)),
        time.strftime("%b", (1, 3, 1, 1, 1, 1, 1, 1, 1)),
        time.strftime("%b", (1, 4, 1, 1, 1, 1, 1, 1, 1)),
        time.strftime("%b", (1, 5, 1, 1, 1, 1, 1, 1, 1)),
        time.strftime("%b", (1, 6, 1, 1, 1, 1, 1, 1, 1)),
        time.strftime("%b", (1, 7, 1, 1, 1, 1, 1, 1, 1)),
        time.strftime("%b", (1, 8, 1, 1, 1, 1, 1, 1, 1)),
        time.strftime("%b", (1, 9, 1, 1, 1, 1, 1, 1, 1)),
        time.strftime("%b", (1, 10, 1, 1, 1, 1, 1, 1, 1)),
        time.strftime("%b", (1, 11, 1, 1, 1, 1, 1, 1, 1)),
        time.strftime("%b", (1, 12, 1, 1, 1, 1, 1, 1, 1)),
    )

    # Gramps day number: Sunday => 1, Monday => 2, etc
    # strftime takes a (from the doc of standard Python library "time")
    #  "tuple or struct_time representing a time as returned by gmtime()
    #   or localtime()"
    # see http://docs.python.org/library/time.html
    # The seventh tuple entry returned by gmtime() is the day-of-the-week
    # number. tm_wday => range [0,6], Monday is 0
    # Note. Only the seventh tuple entry matters. The others are
    # just a dummy.
    _deprecated_long_days = (
        "",
        time.strftime("%A", (1, 1, 1, 1, 1, 1, 6, 1, 1)),  # Sunday
        time.strftime("%A", (1, 1, 1, 1, 1, 1, 0, 1, 1)),  # Monday
        time.strftime("%A", (1, 1, 1, 1, 1, 1, 1, 1, 1)),  # Tuesday
        time.strftime("%A", (1, 1, 1, 1, 1, 1, 2, 1, 1)),  # Wednesday
        time.strftime("%A", (1, 1, 1, 1, 1, 1, 3, 1, 1)),  # Thursday
        time.strftime("%A", (1, 1, 1, 1, 1, 1, 4, 1, 1)),  # Friday
        time.strftime("%A", (1, 1, 1, 1, 1, 1, 5, 1, 1)),  # Saturday
    )

    _deprecated_short_days = (
        "",
        time.strftime("%a", (1, 1, 1, 1, 1, 1, 6, 1, 1)),  # Sun
        time.strftime("%a", (1, 1, 1, 1, 1, 1, 0, 1, 1)),  # Mon
        time.strftime("%a", (1, 1, 1, 1, 1, 1, 1, 1, 1)),  # Tue
        time.strftime("%a", (1, 1, 1, 1, 1, 1, 2, 1, 1)),  # Wed
        time.strftime("%a", (1, 1, 1, 1, 1, 1, 3, 1, 1)),  # Thu
        time.strftime("%a", (1, 1, 1, 1, 1, 1, 4, 1, 1)),  # Fri
        time.strftime("%a", (1, 1, 1, 1, 1, 1, 5, 1, 1)),  # Sat
    )



#
# Gramps - a GTK+/GNOME based genealogy program
#
# Copyright (c) 2016 Gramps Development Team
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
#

"""
Proxy class for the Gramps databases. Caches lookups from handles.
"""

from ..utils.lru import LRU


class CacheProxyDb:
    """
    A Proxy for a database with cached lookups on handles.

    Does not invalid caches. Should be used only in read-only
    places, and not where caches are altered.
    """

    def __init__(self, database):
        """
        CacheProxy will cache items based on their handle.

        Assumes all handles (regardless of type) are unique.
        Database is called self.db for consistency with other
        proxies.
        """
        self.db = database
        # Memory allocation is power of 2 where slots has to fit.
        # LRU uses one extra slot in its work, so set max to 2^17-1
        # otherwise we are just wasting memory
        self.cache_handle = LRU(131071)

    def __del__(self):
        self.cache_handle.clear()

    def __getattr__(self, attr):
        """
        If an attribute isn't found here, use the self.db
        version.
        """
        return getattr(self.db, attr)

    def clear_cache(self, handle=None):
        """
        Clears all caches if handle is None, or
        specific entry.
        """
        if handle:
            del self.cache_handle[handle]
        else:
            self.cache_handle.clear()

    def get_person_from_handle(self, handle):
        """
        Gets item from cache if it exists. Converts
        handles to string, for uniformity.
        """
        if handle not in self.cache_handle:
            self.cache_handle[handle] = self.db.get_person_from_handle(handle)
        return self.cache_handle[handle]

    def get_event_from_handle(self, handle):
        """
        Gets item from cache if it exists. Converts
        handles to string, for uniformity.
        """
        if handle not in self.cache_handle:
            self.cache_handle[handle] = self.db.get_event_from_handle(handle)
        return self.cache_handle[handle]

    def get_family_from_handle(self, handle):
        """
        Gets item from cache if it exists. Converts
        handles to string, for uniformity.
        """
        if handle not in self.cache_handle:
            self.cache_handle[handle] = self.db.get_family_from_handle(handle)
        return self.cache_handle[handle]

    def get_repository_from_handle(self, handle):
        """
        Gets item from cache if it exists. Converts
        handles to string, for uniformity.
        """
        if handle not in self.cache_handle:
            self.cache_handle[handle] = self.db.get_repository_from_handle(handle)
        return self.cache_handle[handle]

    def get_place_from_handle(self, handle):
        """
        Gets item from cache if it exists. Converts
        handles to string, for uniformity.
        """
        if handle not in self.cache_handle:
            self.cache_handle[handle] = self.db.get_place_from_handle(handle)
        return self.cache_handle[handle]

    def get_citation_from_handle(self, handle):
        """
        Gets item from cache if it exists. Converts
        handles to string, for uniformity.
        """
        if handle not in self.cache_handle:
            self.cache_handle[handle] = self.db.get_citation_from_handle(handle)
        return self.cache_handle[handle]

    def get_source_from_handle(self, handle):
        """
        Gets item from cache if it exists. Converts
        handles to string, for uniformity.
        """
        if handle not in self.cache_handle:
            self.cache_handle[handle] = self.db.get_source_from_handle(handle)
        return self.cache_handle[handle]

    def get_note_from_handle(self, handle):
        """
        Gets item from cache if it exists. Converts
        handles to string, for uniformity.
        """
        if handle not in self.cache_handle:
            self.cache_handle[handle] = self.db.get_note_from_handle(handle)
        return self.cache_handle[handle]

    def get_media_from_handle(self, handle):
        """
        Gets item from cache if it exists. Converts
        handles to string, for uniformity.
        """
        if handle not in self.cache_handle:
            self.cache_handle[handle] = self.db.get_media_from_handle(handle)
        return self.cache_handle[handle]

    def get_tag_from_handle(self, handle):
        """
        Gets item from cache if it exists. Converts
        handles to string, for uniformity.
        """
        if handle not in self.cache_handle:
            self.cache_handle[handle] = self.db.get_tag_from_handle(handle)
        return self.cache_handle[handle]



#
# Gramps - a GTK+/GNOME based genealogy program
#
# Copyright (C) 2000-2007  Donald N. Allingham
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
#

"""
SourceMedia types.
"""

# -------------------------------------------------------------------------
#
# Gramps modules
#
# -------------------------------------------------------------------------
from ..const import GRAMPS_LOCALE as glocale
from .grampstype import GrampsType

_ = glocale.translation.gettext


# -------------------------------------------------------------------------
#
# SourceMediaType
#
# -------------------------------------------------------------------------
class SourceMediaType(GrampsType):
    """
    Class encapsulating the media type for a source.
    """

    UNKNOWN = -1
    CUSTOM = 0
    AUDIO = 1
    BOOK = 2
    CARD = 3
    ELECTRONIC = 4
    FICHE = 5
    FILM = 6
    MAGAZINE = 7
    MANUSCRIPT = 8
    MAP = 9
    NEWSPAPER = 10
    PHOTO = 11
    TOMBSTONE = 12
    VIDEO = 13

    _CUSTOM = CUSTOM
    _DEFAULT = BOOK

    _DATAMAP = [
        (UNKNOWN, _("Unknown"), "Unknown"),
        (CUSTOM, _("Custom"), "Custom"),
        (AUDIO, _("Audio"), "Audio"),
        (BOOK, _("Book"), "Book"),
        (CARD, _("Card"), "Card"),
        (ELECTRONIC, _("Electronic"), "Electronic"),
        (FICHE, _("Fiche"), "Fiche"),
        (FILM, _("Film"), "Film"),
        (MAGAZINE, _("Magazine"), "Magazine"),
        (MANUSCRIPT, _("Manuscript"), "Manuscript"),
        (MAP, _("Map"), "Map"),
        (NEWSPAPER, _("Newspaper"), "Newspaper"),
        (PHOTO, _("Photo"), "Photo"),
        (TOMBSTONE, _("Tombstone"), "Tombstone"),
        (VIDEO, _("Video"), "Video"),
    ]

    def __init__(self, value=None):
        GrampsType.__init__(self, value)



#
# Gramps - a GTK+/GNOME based genealogy program
#
# Copyright (C) 2000-2006  Donald N. Allingham
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
#

"""
DateBase class for Gramps.
"""

# -------------------------------------------------------------------------
#
# Gramps modules
#
# -------------------------------------------------------------------------
from .date import Date


# -------------------------------------------------------------------------
#
# DateBase
#
# -------------------------------------------------------------------------
class DateBase:
    """
    Base class for storing date information.
    """

    def __init__(self, source=None):
        """
        Create a new DateBase, copying from source if not None.

        :param source: Object used to initialize the new object
        :type source: DateBase
        """
        if source:
            self.date = Date(source.date)
        else:
            self.date = Date()

    def serialize(self, no_text_date=False):
        """
        Convert the object to a serialized tuple of data.
        """
        if self.date is None or (self.date.is_empty() and not self.date.text):
            date = None
        else:
            date = self.date.serialize(no_text_date)
        return date

    def unserialize(self, data):
        """
        Convert a serialized tuple of data to an object.
        """
        self.date = Date()
        if data is not None:
            self.date.unserialize(data)

    def get_date_object(self):
        """
        Return the :class:`~.date.Date` object associated with the DateBase.

        :returns: Returns a DateBase :class:`~.date.Date` instance.
        :rtype: :class:`~.date.Date`
        """
        if not self.date:
            self.date = Date()
        return self.date

    def set_date_object(self, date):
        """
        Set the :class:`~.date.Date` object associated with the DateBase.

        :param date: :class:`~.date.Date` instance to be assigned to the
                     DateBase
        :type date: :class:`~.date.Date`
        """
        self.date = date



#
# Gramps - a GTK+/GNOME based genealogy program
#
# Copyright (C) 2016 Tom Samstag
# Copyright (C) 2017 Serge Noiraud
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
#

"""
Unittest that tests place-specific filter rules
"""
import unittest
import os

from ....db.utils import import_as_dict
from ....filters import GenericFilterFactory
from ....const import DATA_DIR
from ....user import User

from ..place import (
    AllPlaces,
    HasCitation,
    HasGallery,
    HasIdOf,
    RegExpIdOf,
    HasNote,
    HasNoteRegexp,
    HasReferenceCountOf,
    HasSourceCount,
    HasSourceOf,
    PlacePrivate,
    MatchesSourceConfidence,
    HasData,
    HasNoLatOrLon,
    InLatLonNeighborhood,
    ChangedSince,
    HasTag,
    HasTitle,
    IsEnclosedBy,
    WithinArea,
)

TEST_DIR = os.path.abspath(os.path.join(DATA_DIR, "tests"))
EXAMPLE = os.path.join(TEST_DIR, "example.gramps")
GenericPlaceFilter = GenericFilterFactory("Place")


class BaseTest(unittest.TestCase):
    """
    Place rule tests.
    """

    @classmethod
    def setUpClass(cls):
        """
        Import example database.
        """
        cls.db = import_as_dict(EXAMPLE, User())

    def filter_with_rule(self, rule):
        """
        Apply a filter with the given rule.
        """
        filter_ = GenericPlaceFilter()
        filter_.add_rule(rule)
        results = filter_.apply(self.db)
        return set(results)

    def test_allplaces(self):
        """
        Test AllPlaces rule.
        """
        rule = AllPlaces([])
        self.assertEqual(
            len(self.filter_with_rule(rule)), self.db.get_number_of_places()
        )

    def test_hascitation(self):
        """
        Test HasCitation rule.
        """
        rule = HasCitation(["page 23", "", ""])
        self.assertEqual(self.filter_with_rule(rule), set(["YNUJQC8YM5EGRG868J"]))

    def test_hasgallery(self):
        """
        Test HasGallery rule.
        """
        rule = HasGallery(["0", "greater than"])
        self.assertEqual(self.filter_with_rule(rule), set(["YNUJQC8YM5EGRG868J"]))

    def test_hasidof(self):
        """
        Test HasIdOf rule.
        """
        rule = HasIdOf(["P0001"])
        self.assertEqual(
            self.filter_with_rule(rule), set(["c96587262e91149933fcea5f20a"])
        )

    def test_regexpidof(self):
        """
        Test RegExpIdOf rule.
        """
        rule = RegExpIdOf(["P000."], use_regex=True)
        self.assertEqual(
            self.filter_with_rule(rule),
            set(
                [
                    "c96587262e91149933fcea5f20a",
                    "c96587262ff262aaac31f6db7af",
                    "c96587262f24c33ab2420276737",
                    "c96587262e566596a225682bf53",
                    "c9658726302661576894508202d",
                    "c96587262f8329d37b252e1b9e5",
                    "c965872630664f33485fc18e75",
                    "c96587262fb7dbb954077cb1286",
                    "c96587262f4a44183c65ff1e52",
                    "c96587262ed43fdb37bf04bdb7f",
                ]
            ),
        )

    def test_hasnote(self):
        """
        Test HasNote rule.
        """
        rule = HasNote([])
        self.assertEqual(self.filter_with_rule(rule), set([]))

    def test_hasnoteregexp(self):
        """
        Test HasNoteRegexp rule.
        """
        rule = HasNoteRegexp(["."], use_regex=True)
        self.assertEqual(self.filter_with_rule(rule), set([]))

    def test_hasreferencecountof(self):
        """
        Test HasReferenceCountOf rule.
        """
        rule = HasReferenceCountOf(["greater than", "35"])
        self.assertEqual(
            self.filter_with_rule(rule),
            set(
                [
                    "c96587262e566596a225682bf53",
                    "MATJQCJYH8ULRIRYTH",
                    "5HTJQCSB91P69HY731",
                    "4ECKQCWCLO5YIHXEXC",
                    "c965872630a68ebd32322c4a30a",
                ]
            ),
        )

    def test_hassourcecount(self):
        """
        Test HasSourceCount rule.
        """
        rule = HasSourceCount(["1", "equal to"])
        self.assertEqual(self.filter_with_rule(rule), set(["YNUJQC8YM5EGRG868J"]))

    def test_hassourceof(self):
        """
        Test HasSourceOf rule.
        """
        rule = HasSourceOf(["S0001"])
        self.assertEqual(self.filter_with_rule(rule), set(["YNUJQC8YM5EGRG868J"]))

    def test_placeprivate(self):
        """
        Test PlacePrivate rule.
        """
        rule = PlacePrivate([])
        self.assertEqual(self.filter_with_rule(rule), set([]))

    def test_matchessourceconfidence(self):
        """
        Test MatchesSourceConfidence rule.
        """
        rule = MatchesSourceConfidence(["2"])
        self.assertEqual(self.filter_with_rule(rule), set(["YNUJQC8YM5EGRG868J"]))

    def test_hasdata(self):
        """
        Test HasData rule.
        """
        rule = HasData(["Albany", "County", ""])
        self.assertEqual(
            self.filter_with_rule(rule), set(["c9658726d602acadb74e330116a"])
        )

    def test_hasnolatorlon(self):
        """
        Test HasNoLatOrLon rule.
        """
        rule = HasNoLatOrLon([])
        self.assertEqual(len(self.filter_with_rule(rule)), 921)

    def test_inlatlonneighborhood(self):
        """
        Test InLatLonNeighborhood rule.
        """
        rule = InLatLonNeighborhood(["30N", "90W", "2", "2"])
        self.assertEqual(
            self.filter_with_rule(rule),
            set(
                [
                    "C6WJQC0GDYP3HZDPR3",
                    "N88LQCRB363ES5WJ83",
                    "03EKQCC2KTNLHFLDRJ",
                    "M9VKQCJV91X0M12J8",
                ]
            ),
        )

    def test_changedsince(self):
        """
        Test ChangedSince rule.
        """
        rule = ChangedSince(["2013-01-01", "2014-01-01"])
        self.assertEqual(len(self.filter_with_rule(rule)), 419)

    def test_hastag(self):
        """
        Test HasTag rule.
        """
        rule = HasTag(["ToDo"])
        self.assertEqual(self.filter_with_rule(rule), set([]))

    def test_hastitle(self):
        """
        Test HasTitle rule.
        """
        rule = HasTitle(["Albany"])
        self.assertEqual(
            self.filter_with_rule(rule),
            set(
                [
                    "51VJQCXUP61H9JRL66",
                    "9XBKQCE1LZ7PMJE56G",
                    "c9658726d602acadb74e330116a",
                    "P9MKQCT08Z3YBJV5UB",
                ]
            ),
        )

    def test_isenclosedby(self):
        """
        Test IsEnclosedBy rule.
        """
        rule = IsEnclosedBy(["P0001", "0"])
        self.assertEqual(
            self.filter_with_rule(rule),
            set(
                [
                    "EAFKQCR0ED5QWL87EO",
                    "S22LQCLUZM135LVKRL",
                    "VDUJQCFP24ZV3O4ID2",
                    "V6ALQCZZFN996CO4D",
                    "OC6LQCXMKP6NUVYQD8",
                    "CUUKQC6BY5LAZXLXC6",
                    "PTFKQCKPHO2VC5SYKS",
                    "PHUJQCJ9R4XQO5Y0WS",
                ]
            ),
        )

    def test_withinarea(self):
        """
        Test within area rule.
        """
        rule = WithinArea(["P1339", 100, 0])
        self.assertEqual(
            self.filter_with_rule(rule),
            set(
                [
                    "KJUJQCY580EB77WIVO",
                    "TLVJQC4FD2CD9OYAXU",
                    "TE4KQCL9FDYA4PB6VW",
                    "W9GLQCSRJIQ9N2TGDF",
                ]
            ),
        )

    def test_isenclosedby_inclusive(self):
        """
        Test IsEnclosedBy rule with inclusive option.
        """
        rule = IsEnclosedBy(["P0001", "1"])
        self.assertEqual(
            self.filter_with_rule(rule),
            set(
                [
                    "c96587262e91149933fcea5f20a",
                    "EAFKQCR0ED5QWL87EO",
                    "S22LQCLUZM135LVKRL",
                    "VDUJQCFP24ZV3O4ID2",
                    "V6ALQCZZFN996CO4D",
                    "OC6LQCXMKP6NUVYQD8",
                    "CUUKQC6BY5LAZXLXC6",
                    "PTFKQCKPHO2VC5SYKS",
                    "PHUJQCJ9R4XQO5Y0WS",
                ]
            ),
        )


if __name__ == "__main__":
    unittest.main()



# -*- coding: utf-8 -*-
#!/usr/bin/python
#
# Gramps - a GTK+/GNOME based genealogy program
#
# Copyright (C) 2000-2007  Donald N. Allingham
# Copyright (C) 2007       Johan Gonqvist <johan.gronqvist@gmail.com>
# Copyright (C) 2007       Gary Burton <gary.burton@zen.co.uk>
# Copyright (C) 2007-2009  Stephane Charette <stephanecharette@gmail.com>
# Copyright (C) 2008       Brian G. Matherly
# Copyright (C) 2008       Jason M. Simanek <jason@bohemianalps.com>
# Copyright (C) 2008-2011  Rob G. Healey <robhealey1@gmail.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
#

"""
General constants used in different html enabled plugins
"""

from gramps.gen.const import GRAMPS_LOCALE as glocale

_ = glocale.translation.gettext

# ------------------------------------------------------------------------
#
# Gramps modules
#
# ------------------------------------------------------------------------

# ------------------------------------------------------------------------
#
# Constants
#
# ------------------------------------------------------------------------

_CHARACTER_SETS = [
    # First is used as default selection.
    # As seen on the internet, ISO-xxx are listed as capital letters
    [_("Unicode UTF-8 (recommended)"), "UTF-8"],
    ["ISO-8859-1", "ISO-8859-1"],
    ["ISO-8859-2", "ISO-8859-2"],
    ["ISO-8859-3", "ISO-8859-3"],
    ["ISO-8859-4", "ISO-8859-4"],
    ["ISO-8859-5", "ISO-8859-5"],
    ["ISO-8859-6", "ISO-8859-6"],
    ["ISO-8859-7", "ISO-8859-7"],
    ["ISO-8859-8", "ISO-8859-8"],
    ["ISO-8859-9", "ISO-8859-9"],
    ["ISO-8859-10", "ISO-8859-10"],
    ["ISO-8859-13", "ISO-8859-13"],
    ["ISO-8859-14", "ISO-8859-14"],
    ["ISO-8859-15", "ISO-8859-15"],
    [
        "koi8_r",
        "koi8_r",
    ],
]

_CC = [
    "",
    '<a rel="license" href="http://creativecommons.org/licenses/by/4.0/">'
    '<img alt="Creative Commons License - By attribution" '
    'title="Creative Commons License - By attribution" '
    'src="%(gif_fname)s" /></a>',
    '<a rel="license" href="http://creativecommons.org/licenses/by-nd/4.0/">'
    '<img alt="Creative Commons License - By attribution, No derivations" '
    'title="Creative Commons License - By attribution, No derivations" '
    'src="%(gif_fname)s" /></a>',
    '<a rel="license" href="http://creativecommons.org/licenses/by-sa/4.0/">'
    '<img alt="Creative Commons License - By attribution, Share-alike" '
    'title="Creative Commons License - By attribution, Share-alike" '
    'src="%(gif_fname)s" /></a>',
    '<a rel="license" href="http://creativecommons.org/licenses/by-nc/4.0/">'
    '<img alt="Creative Commons License - By attribution, Non-commercial" '
    'title="Creative Commons License - By attribution, Non-commercial" '
    'src="%(gif_fname)s" /></a>',
    '<a rel="license" href="http://creativecommons.org/licenses/by-nc-nd/4.0/">'
    '<img alt="Creative Commons License - By attribution, Non-commercial, '
    'No derivations" '
    'title="Creative Commons License - By attribution, Non-commercial, '
    'No derivations" '
    'src="%(gif_fname)s" /></a>',
    '<a rel="license" href="http://creativecommons.org/licenses/by-nc-sa/4.0/">'
    '<img alt="Creative Commons License - By attribution, Non-commerical, '
    'Share-alike" '
    'title="Creative Commons License - By attribution, Non-commerical, '
    'Share-alike" '
    'src="%(gif_fname)s" /></a>',
]

_COPY_OPTIONS = [
    _("Standard copyright"),
    # This must match _CC
    # translators, long strings, have a look at Web report dialogs
    _("Creative Commons - By attribution"),
    _("Creative Commons - By attribution, No derivations"),
    _("Creative Commons - By attribution, Share-alike"),
    _("Creative Commons - By attribution, Non-commercial"),
    _("Creative Commons - By attribution, Non-commercial, No derivations"),
    _("Creative Commons - By attribution, Non-commercial, Share-alike"),
    _("No copyright notice"),
]



#
# Gramps - a GTK+/GNOME based genealogy program
#
# Copyright (C) 2002-2007  Donald N. Allingham
# Copyright (C) 2007-2008  Brian G. Matherly
# Copyright (C) 2011       Tim G L Lyons
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
#

"""
Package providing filter rules for Gramps.
"""

from ._hascitation import HasCitation
from ._allcitations import AllCitations
from ._changedsince import ChangedSince
from ._citationprivate import CitationPrivate
from ._hasattribute import HasAttribute
from ._hasgallery import HasGallery
from ._hasidof import HasIdOf
from ._hasnote import HasNote
from ._hasnotematchingsubstringof import HasNoteMatchingSubstringOf
from ._hasnoteregexp import HasNoteRegexp
from ._hasnotetype import HasNoteType
from ._hasreferencecountof import HasReferenceCountOf
from ._hassource import HasSource
from ._hassourceidof import HasSourceIdOf
from ._hassourcenoteregexp import HasSourceNoteRegexp
from ._matchesfilter import MatchesFilter
from ._matchespagesubstringof import MatchesPageSubstringOf
from ._matchesrepositoryfilter import MatchesRepositoryFilter
from ._matchessourcefilter import MatchesSourceFilter
from ._regexpidof import RegExpIdOf
from ._regexpsourceidof import RegExpSourceIdOf
from ._hastag import HasTag

editor_rule_list = [
    HasCitation,
    AllCitations,
    ChangedSince,
    CitationPrivate,
    HasAttribute,
    HasGallery,
    HasIdOf,
    HasNote,
    HasNoteRegexp,
    HasNoteType,
    HasReferenceCountOf,
    HasSource,
    HasSourceIdOf,
    HasSourceNoteRegexp,
    MatchesFilter,
    MatchesPageSubstringOf,
    MatchesRepositoryFilter,
    MatchesSourceFilter,
    RegExpIdOf,
    RegExpSourceIdOf,
    HasTag,
]
